-- 半月礼包
Lua_HalfMonthGiftPanel = class("Lua_HalfMonthGiftPanel", Lua_DisplayObject);
function Lua_HalfMonthGiftPanel:ctor(p, skinName)
	self:superCall("ctor", p, skinName);
end

function Lua_HalfMonthGiftPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.desTxt_1:setLangValue(789000);
	self.desTxt_3:setLangValue(789001);
	self.desTxt_5:setLangValue(789002);

	self.btnList = Lua_DragList.new(self.content, 692, 79, Lua_AxisDefine.Horizontal);
	self.btnList:setListCell(Lua_HalfMonthGiftBtnItem, 170, 79, 1);
	self.btnList:setRectAnchor(1);
	self.btnList:setLocation(0, -380, 0);

	self.ruleBtn:addClickFuntion(self, self.onRuleClick);
	self.previewAllBtn:addClickFuntion(self, self.onPreviewAllClick);
	self.buyBtn:addClickFuntion(self, self.onBuyClick);

	self:initShowInfo();

	self:addLuaEvent("UPDATE_HALFMONTHGIFT_INFO", self, self.updateHalfMonthGiftInfo);
	self:addLuaEvent("ON_HALFMONTHGIFT_BTN_CLICK", self, self.onBtnClick);
	self:addLuaEvent("ON_HALFMONTHGIFT_OPTIONAL_CLICK", self, self.onOptionalRewardClick);
	self:addLuaEvent("ON_HALFMONTHGIFT_FIX_CLICK", self, self.onFixRewardClick);
end

function Lua_HalfMonthGiftPanel:onScreenResize(adaptHeight)
	self:superCall("onScreenResize", adaptHeight);
	self:initShowInfo();
	if self.dragContainer ~= nil then
		self.dragContainer:setContentRect(638, 750 + adaptHeight);
	end
end

function Lua_HalfMonthGiftPanel:initShowInfo()
	if self.adaptHeight < 0 then
		self.dayTipsBg:setLocation(-178, -256, 0);
		self.dayTipsTxt:setLocation(-265, -254, 0);
		self.previewAllBtn:setLocation(219, -254, 0);
		self.buyBtn:setLocation(0, -330, 0);
	else
		self.dayTipsBg:setLocation(-178, -343, 0);
		self.dayTipsTxt:setLocation(-265, -341, 0);
		self.previewAllBtn:setLocation(219, -341, 0);
		self.buyBtn:setLocation(0, -417, 0);
	end
end

function Lua_HalfMonthGiftPanel:setData(actVo)
	self.actVo = actVo;
	self:startTimer();
	self:updateLeftTime();
end

function Lua_HalfMonthGiftPanel:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self:addTimer("Lua_HalfMonthGiftPanel", 1, self, self.updateLeftTime);
	end
end

function Lua_HalfMonthGiftPanel:updateLeftTime()
	local time = self.actVo.residueTime;
	if time <= 0 then
		time = 0;
		self:endTimer();
	end
	self.timeTxt.text = Lua_TimeTransform.transSeconds20(time, true, true);
end

function Lua_HalfMonthGiftPanel:endTimer()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.updateLeftTime);
	end
end

function Lua_HalfMonthGiftPanel:updateHalfMonthGiftInfo()
	local giftTypeArr = DataCenter.halfMonthGiftData.giftTypeArr;
	local arr = Array();
	for i = 1, giftTypeArr:getSize() do
		arr:add(giftTypeArr[i]);
	end
	arr:sortf(self, self.sortFun);
	local judgeType = self.giftType or arr[1];
	local toIndex = arr:indexOf(judgeType);
	self.btnList.canDrag = arr:getSize() > 4;
	self.btnList:setEnable(arr:getSize() > 1);
	self.btnList:setArrayData(arr);
	self.btnList:setSelectClick(toIndex);
	self.btnList:setSelected(toIndex);
end

function Lua_HalfMonthGiftPanel:sortFun(a, b)
	local vo_a = DataCenter.halfMonthGiftData:getHalfMonthGiftByGifType(a);
	local vo_b = DataCenter.halfMonthGiftData:getHalfMonthGiftByGifType(b);
	if vo_a.recharge_id == vo_a.recharge_id then
		if vo_a.recharge_state == vo_b.recharge_state then
			return vo_a.gift_id < vo_b.gift_id;
		else
			return vo_a.recharge_state == 1;
		end
	else
		return vo_a.recharge_id <= 0;
	end
end

function Lua_HalfMonthGiftPanel:onBtnClick(info)
	self.giftType = info.gift_type;

	self.desTxt_2.text = info.gift_title;
	self.desTxt_4.text = #info.reward_list;
	self.desTxt_2:resetSize();
	self.desTxt_3:resetSize();
	self.desTxt_4:resetSize();
	self.desTxt_3:setLocation(self.desTxt_2.posX + self.desTxt_2.width + 6, -265, 0);
	self.desTxt_4:setLocation(self.desTxt_3.posX + self.desTxt_3.width + 5, -265, 0);
	self.desTxt_5:setLocation(self.desTxt_4.posX + self.desTxt_4.width + 3, -265, 0);

	self.dayTipsTxt.text = LanUtil.formateStr(789003, DataCenter.halfMonthGiftData.curDay);

	if info.recharge_id > 0 then
		local rechargeInfo = Lua_RechargeConfigData.getRechargeConfig(info.recharge_id);
		self.buyBtn.label = LanUtil.formateStr(789004, tonumber(rechargeInfo.money));
	end
	self.previewObj:SetActive(info.recharge_state == 0 and info.recharge_id > 0); -- 0未解锁 1已解锁
	self:updatePreviewRewardInfo(info.recharge_state == 0 and info.recharge_id > 0);
	self:updateShowRewardInfo(info.recharge_state ~= 0 or info.recharge_id <= 0);
end

function Lua_HalfMonthGiftPanel:updatePreviewRewardInfo(state)
	if state then
		if self.optionalItem == nil then
			self.optionalItem = Lua_HalfMonthGiftItem.new(self.previewObj);
			self.optionalItem:setLocation(0, 144, 0);
			self.optionalItem:initRewardType(Lua_HalfMonthGiftData.optionalType);
			self.fixItem = Lua_HalfMonthGiftItem.new(self.previewObj);
			self.fixItem:setLocation(0, -112, 0);
			self.fixItem:initRewardType(Lua_HalfMonthGiftData.fixType);
		end
		self.optionalItem:setData(self.giftType); -- 自选奖励
		self.fixItem:setData(self.giftType); -- 固定奖励
	end
end

function Lua_HalfMonthGiftPanel:updateShowRewardInfo(state)
	if state then
		if self.dragContainer == nil then
			self.dragContainer = Lua_DragListContainer.new(self.content, 638, 750 + self.adaptHeight, Lua_AxisDefine.Vertical);
			self.dragContainer:setCellSpacing(10, 15);
			self.dragContainer:setLocation(2.5, -146, 0);
		end
		self.dragContainer:setEnable(true);
		self:setShowRewardInfo();
	else
		if self.dragContainer ~= nil then
			self.dragContainer:setEnable(false);
		end
	end
end

function Lua_HalfMonthGiftPanel:setShowRewardInfo()
	local list = DataCenter.halfMonthGiftData:getPreviewInfoByGifType(self.giftType);
	local toIndex;
	self.dragContainer:clearList();
	for i = 1, #list do
		for j = 1, list[i]:getSize() do
			if (toIndex == nil or toIndex > list[i][j].day_index) and (list[i][j].state == 1 or list[i][j].state == 0) then
				toIndex = list[i][j].day_index;
			end
		end
		if list[i][1].reward_type == 2 then
			self.dragContainer:addChildData(list[i], Lua_HalfMonthGiftOptionalItem, 638, 186, 1); -- 自选奖励
		else
			self.dragContainer:addChildData(list[i], Lua_HalfMonthGiftFixItem, 206, 186, 3); -- 固定奖励
		end
	end
	self.dragContainer:updateChildPos();
	self.dragContainer:moveToIndex(toIndex or 1, true, false);
end

function Lua_HalfMonthGiftPanel:onRuleClick(go)
	local tipData = { des = self.actVo.help_des, posX = 195, posY = -245 };
	Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipData);
end

function Lua_HalfMonthGiftPanel:onPreviewAllClick(go)
	self:onHalfMonthGiftOperate(2);
end

function Lua_HalfMonthGiftPanel:onBuyClick(go)
	self:onHalfMonthGiftOperate(3);
end

function Lua_HalfMonthGiftPanel:onOptionalRewardClick(param)
	self:onHalfMonthGiftOperate(4, param);
end

function Lua_HalfMonthGiftPanel:onFixRewardClick(param)
	self:onHalfMonthGiftOperate(5, param);
end

function Lua_HalfMonthGiftPanel:onHalfMonthGiftOperate(operateType, param)
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end

	local info = DataCenter.halfMonthGiftData:getHalfMonthGiftByGifType(self.giftType);
	if operateType == 1 then -- 刷新
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(self.actVo.actIdx);
	elseif operateType == 2 then -- 预览全部奖励
		Lua_WindowUtil.openWin("Lua_HalfMonthGiftPreviewWindow", Lua_WinParam.ACTION_OPEN, self.giftType);
	elseif operateType == 3 then -- 购买礼包
		local rechargeInfo = Lua_RechargeConfigData.getRechargeConfig(info.recharge_id);
		DataCenter.sdkInterface:PointOut(1, tonumber(rechargeInfo.money), tonumber(rechargeInfo.money_type),
			tonumber(rechargeInfo.id), rechargeInfo.gift_name, rechargeInfo.gold);
	elseif operateType == 4 then -- 选择自选奖励
		Lua_WindowUtil.openWin("Lua_HalfMonthGiftSelectWindow", Lua_WinParam.ACTION_OPEN, {actVo = self.actVo, giftInfo = param});
	elseif operateType == 5 then -- 选择自选奖励
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sDrawHalfMonthGift(self.actVo.actIdx, param.id);
	end
end

function Lua_HalfMonthGiftPanel:dispose()
	if self.btnList ~= nil then
		self.btnList:dispose();
		self.btnList = nil;
	end
	self.actVo = nil;
	self:endTimer();
	self:superCall("dispose");
end










Lua_HalfMonthGiftBtnItem = class("Lua_HalfMonthGiftBtnItem", Lua_ListCell);
function Lua_HalfMonthGiftBtnItem:ctor(p)
	self:superCall("ctor", p);
end

function Lua_HalfMonthGiftBtnItem:buildShowItem()
	self:superCall("buildShowItem");

	self.btnBg = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
	self.btnBg.anchorValue = 3;
	self.btnBg.pivotValue = 3;
	self.btnBg.clickScale = false;

	self.btnTxt = UICreate.createLabel(self.content, nil, 0, 0, 100, 31, 26);
	self.btnTxt.pivotValue = 4;
	self.btnTxt.alignmentValue = 4;
	self.btnTxt.horizontalOverflowValue = 1;
	self.btnTxt.effectStyle = 2;
	self.btnTxt:setEffectDistance(2, 2);
end

function Lua_HalfMonthGiftBtnItem:setData(data)
	self._data = DataCenter.halfMonthGiftData:getHalfMonthGiftByGifType(data);
	if data == nil or self._data == nil then
		return;
	end

	self.btnTxt.text = self._data.gift_name;
	self:checkRedPoint();
end

function Lua_HalfMonthGiftBtnItem:checkRedPoint()
	if self._data ~= nil then
		local redPointId = "HalfMonthGift_" .. self._data.gift_type;
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content, redPointId);
			self.redPoint:setLocation(66, 22);
		end
		self.redPoint:changeRedPoint(redPointId);
	else
		if self.redPoint ~= nil then
			self.redPoint:setEnable(false);
		end
	end
end

function Lua_HalfMonthGiftBtnItem:onPressClick()
	if self._data ~= nil then
		Lua_MsgUtil.callFunc("ON_HALFMONTHGIFT_BTN_CLICK", self._data);
		return true;
	end
end

function Lua_HalfMonthGiftBtnItem:onSelectionChange(bSelect)
	self:superCall("onSelectionChange", bSelect);
	if bSelect then
		self.btnBg:setAltas("half_month_gift_atlas", "xui_bylb_tab_02");
	    self.btnTxt.effectColorStr = Lua_ColorDefine.transColor("C85627");
	else
		self.btnBg:setAltas("half_month_gift_atlas", "xui_bylb_tab_01");
	    self.btnTxt.effectColorStr = Lua_ColorDefine.transColor("B46941");
	end
end

function Lua_HalfMonthGiftBtnItem:dispose()
	if self.btnBg ~= nil then
		self.btnBg:dispose();
		self.btnBg = nil;
	end
	if self.btnTxt ~= nil then
		self.btnTxt:dispose();
		self.btnTxt = nil;
	end
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self:superCall("dispose");
end










Lua_HalfMonthGiftItem = class("Lua_HalfMonthGiftItem", Lua_DisplayObject); -- Lua_ListCell);
function Lua_HalfMonthGiftItem:ctor(p)
	self:superCall("ctor", p, "HalfMonthGiftItem_skin");
end

function Lua_HalfMonthGiftItem:buildShowItem()
	self:superCall("buildShowItem");

	self.rewardList = Lua_DragList.new(self.content, 520, 100, Lua_AxisDefine.Horizontal);
	self.rewardList:setListCell(Lua_HalfMonthGiftRewardItemm, 100, 100, 1);
	self.rewardList:setCellSpacing(10, 0);
	self.rewardList:setRectPivot(3);
	self.rewardList:setLocation(-262, -30, 0);
	-- self:setSubList(self.rewardList);
end

function Lua_HalfMonthGiftItem:initRewardType(rewardType)
	self.rewardType = rewardType;
	self.titleTxt:setLangValue(rewardType == 2 and 789005 or 789006);
end

function Lua_HalfMonthGiftItem:setData(data)
	if data == nil then
		return;
	end

	local info = DataCenter.halfMonthGiftData:getHalfMonthGiftByGifType(data);
	local obj, tvoInfo, tvo;
	local itemDic = {};
	local arr = Array();
	for i = 1, #info.reward_list do
		obj = info.reward_list[i];
		if obj.reward_type == self.rewardType then
			for j = 1, #obj.item_list do
				tvoInfo = itemDic[obj.item_list[j].id];
				if tvoInfo == nil then
					tvoInfo = {id = obj.item_list[j].id, amount = obj.item_list[j].amount};
					itemDic[obj.item_list[j].id] = tvoInfo;
				else
					tvoInfo.amount = tvoInfo.amount + obj.item_list[j].amount;
				end
			end
		end
	end
	for k, v in pairs(itemDic) do
		tvo = Lua_ToolVo.createToolVo(k);
		tvo:fromServer(v);
		arr:add(tvo);
	end
	arr:sort("sortWeight", false);
	self.rewardList.canDrag = arr:getSize() > 4;
	self.rewardList:setArrayData(arr);
end

function Lua_HalfMonthGiftItem:dispose()
	if self.rewardList ~= nil then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	self:superCall("dispose");
end










Lua_HalfMonthGiftRewardItemm = class("Lua_HalfMonthGiftRewardItemm", Lua_ItemIconCell);
function Lua_HalfMonthGiftRewardItemm:buildShowItem()
	self:superCall("buildShowItem");
	self.itemIcon:setScale(0.8, 0.8);
end










Lua_HalfMonthGiftOptionalItem = class("Lua_HalfMonthGiftOptionalItem", Lua_DragContainerItem)
function Lua_HalfMonthGiftOptionalItem:ctor(p)
	self:superCall("ctor", p, "HalfMonthGiftOptionalItem_skin");
end

function Lua_HalfMonthGiftOptionalItem:buildShowItem()
	self:superCall("buildShowItem");

	self.rewardList = Lua_DragList.new(self.content, 378, 81, Lua_AxisDefine.Horizontal);
	-- self.rewardList.canLongPress = true;
	-- self.rewardList.longPressLimit = 0.4;
	self.rewardList:setListCell(Lua_HalfMonthGiftOptionalRewardItemm, 81, 81, 1);
	self.rewardList:setCellSpacing(10, 0);
	self.rewardList:setRectPivot(3);
	self.rewardList:setSiblingIndex(1);
	self.rewardList:setLocation(-90, -12, 0);
	self:setSubList(self.rewardList);
end

function Lua_HalfMonthGiftOptionalItem:setData(data)
	if data == nil then
		return;
	end

	self.dayTxt.text = data.title;
	self.tipsTxt.text = LanUtil.formateStr(789007, convert_arab_to_chinese(data.free_number, true));
	self.desTxt.text = data.content;
	self.itemIcon:setAltas(data.icon_config.icon_atlas, data.icon_config.icon_index);

	local dic = {};
	for i = 1, #data.free_item_list do
		dic[data.free_item_list[i].key] = data.free_item_list[i].value;
	end

	local arr = Array()
	for i = 1, #data.item_list do
		tvo = Lua_ToolVo.createToolVo(data.item_list[i].id);
		tvo:fromServer(data.item_list[i]);
		tvo.giftInfo = data;
		tvo.isDraw = dic[data.item_list[i].id] and dic[data.item_list[i].id] == tvo.count;
		arr:add(tvo);
	end
	self.rewardList.canDrag = arr:getSize() > 4;
	self.rewardList:setArrayData(arr);

	self.maskIcon:setEnable(data.state == 2);
	self.drawIcon:setEnable(data.state == 2);
end

function Lua_HalfMonthGiftOptionalItem:dispose()
	if self.rewardList then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	self:superCall("dispose");
end










Lua_HalfMonthGiftOptionalRewardItemm = class("Lua_HalfMonthGiftOptionalRewardItemm", Lua_ItemIconCell);
function Lua_HalfMonthGiftOptionalRewardItemm:buildShowItem()
	self:superCall("buildShowItem");
	self.itemIcon:setScale(0.65, 0.65);
end

function Lua_HalfMonthGiftOptionalRewardItemm:setData(data)
	self:superCall("setData", data);

	self.itemIcon:setDrawMask(data.isDraw);
	self.itemIcon:showRedPoint(data.giftInfo.state == 1);
end

function Lua_HalfMonthGiftOptionalRewardItemm:onPressClick()
	if self.data == nil then
		return;
	end
	if self.data.giftInfo.state == 1 then -- 0没有奖励 1可领取 2已领取
		Lua_MsgUtil.callFunc("ON_HALFMONTHGIFT_OPTIONAL_CLICK", self.data.giftInfo);
	else
		DataCenter.packUtil:showItemInfo(self.data);
	end
end

-- function Lua_HalfMonthGiftOptionalRewardItemm:onLongPressClick()
-- 	if self.data == nil then
-- 		return;
-- 	end
-- 	DataCenter.packUtil:showItemInfo(self.data);
-- end










Lua_HalfMonthGiftFixItem = class("Lua_HalfMonthGiftFixItem", Lua_DragContainerItem)
function Lua_HalfMonthGiftFixItem:ctor(p)
	self:superCall("ctor", p, "HalfMonthGiftFixItem_skin");
end

function Lua_HalfMonthGiftFixItem:buildShowItem()
	self:superCall("buildShowItem");

	self.itemIcon = Lua_ItemIcon.new(self.content);
	self.itemIcon:setCanClick(false);
	self.itemIcon:setSiblingIndex(2);
	self.itemIcon:setScale(0.7, 0.7);
	self.itemIcon:setLocation(0, -12.5, 0);
end

function Lua_HalfMonthGiftFixItem:setData(data)
	self._data = data;
	if data == nil then
		return;
	end

	self.dayTxt.text = data.title;
	local tvo = Lua_ToolVo.createToolVo(data.item_list[1].id);
	tvo:fromServer(data.item_list[1]);
	self.itemIcon:setData(tvo);
	self.itemIcon:showRedPoint(data.state == 1);

	self.maskIcon:setEnable(data.state == 2);
	self.drawIcon:setEnable(data.state == 2);
end

function Lua_HalfMonthGiftFixItem:checkInBound(posX, posY)
	local minX = self.centerPosX - self.itemIcon.itemW / 2 * self.itemIcon.scaleX;
	local maxX = self.centerPosX + self.itemIcon.itemW / 2 * self.itemIcon.scaleX;
	local minY = self.centerPosY - self.itemIcon.itemH / 2 * self.itemIcon.scaleY;
	local maxY = self.centerPosY + self.itemIcon.itemH / 2 * self.itemIcon.scaleY;
	return posX >= minX and posX <= maxX and posY >= minY and posY <= maxY;
end

function Lua_HalfMonthGiftFixItem:onPressClick()
	if self._data == nil then
		return;
	end
	if self._data.state == 1 then -- 0没有奖励 1可领取 2已领取
		Lua_MsgUtil.callFunc("ON_HALFMONTHGIFT_FIX_CLICK", self._data);
	else
		self.itemIcon:onItemClick();
	end
end

function Lua_HalfMonthGiftFixItem:dispose()
	if self.itemIcon then
		self.itemIcon:dispose();
		self.itemIcon = nil;
	end
	self._data = nil;
	self:superCall("dispose");
end