-- 豪礼兑换 panel
Lua_LuxuryCashPanel = class("Lua_LuxuryCashPanel", Lua_DisplayObject);

function Lua_LuxuryCashPanel:ctor(p, skinName)
	self:superCall("ctor", p, skinName);
end

function Lua_LuxuryCashPanel:buildPanel()
	self:superCall("buildPanel");
	self:addLuaEvent("UPDATE_LUXURYCASH_INFO", self, self.onUpdateInfo); 

	self:addLuaEvent("MONEY_UPDATE", self, self.onItemUpdate); 
	self:addLuaEvent("PACK_ITEMS_UPDATE", self, self.onItemUpdate);
end

function Lua_LuxuryCashPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.redPoint = Lua_RedPointIcon.new(self.content, Lua_RedPointList.LuxuryCash);
	self.redPoint:setLocation(102, 259, 0);
	self.redPoint:setRectAnchor(7);

	self.textArea = Lua_TextArea.new(self.content, 600, 140);
	self.textArea:setLocation(-16, 631);
	self.textArea:setRectAnchor(7);

	self.rewardList = Lua_DragList.new(self.content, 600, 120, Lua_AxisDefine.Horizontal);
	self.rewardList:setListCell(Lua_LuxuryCashItem, 120, 120, 1);
	self.rewardList:setCellSpacing(10, 0);
	self.rewardList:setRectAnchor(7);
	self.rewardList:setLocation(2, 388);

	self.addBtn:addClickFuntion(self, self.onAddBtnClick);
	self.goBtn:addClickFuntion(self, self.onGoBtnClick);
end

function Lua_LuxuryCashPanel:dispose()
	if self.textArea ~= nil then
		self.textArea:dispose();
		self.textArea = nil;
	end
	if self.rewardList ~= nil then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self.actVo = nil
	self:superCall("dispose");
end

function Lua_LuxuryCashPanel:setData(actVo)
	self.actVo = actVo;
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sGetLuxuryCashTaskInfo(actVo.actIdx);
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sLuxuryCashGiftInfo(actVo.actIdx);
end

function Lua_LuxuryCashPanel:onUpdateInfo()
	local data = DataCenter.luxuryCashData;
	if data.actIdx ~= self.actVo.actIdx then return end

	local itemVo = data:getCostItemVo(Lua_LuxuryCashData.norType);
	self.costId = itemVo.id;
	self.rewardIcon:setAltas(itemVo.atlas, itemVo.sprite);
	self.itemIcon:setAltas(itemVo.iconAtlas, itemVo.iconSprite);
	self.itemTxt.text = numberTrans(DataCenter.packData:getItemCountById(self.costId));

	self.textArea:setText(self.actVo.help_des, 24, 1.1);

	local rewards = data.rewardList or {};
	local distance = 120 + 10; 
	local recWidth = #rewards <= 4 and #rewards * distance - 10 or distance * 4.5 - 10 * 0.5;
	self.rewardList.canDrag = #rewards > 4;
	self.rewardList:setContentRect(recWidth, 120, false);
	self.rewardList:setArrayData(rewards);

	self.time = data.closeTime - DataCenter.roleData.serverTime;
	self.addBtn:setEnable(self.time > 0);
	self:onTimer();
end

function Lua_LuxuryCashPanel:onTimer()
	if self.time > 0 then
		if not self.isTimer then
			self.isTimer = true;
			self:addTimer("Lua_LuxuryCashPanel_onTimer", 1, self, self.onTimer);
		end
		self.timeTxt.text = LanUtil.formateStr(795000, Lua_TimeTransform.transSeconds16(self.time, true, true, true, false));
		self.time = self.time - 1;
	else
		if self.isTimer then
			self.isTimer = false;
			self:removeTimer(self, self.onTimer);
			self.addBtn:setEnable(false);
		end
		self.timeTxt:setLangValue(795001);
	end
end

function Lua_LuxuryCashPanel:onItemUpdate(dic)
	if self.costId == nil or dic == nil then return end

	local list = DataCenter.luxuryCashData.costList;
	local itemId;
	for i = 1, #list do
		itemId = list[i].value;
		if dic[itemId] ~= nil then
			DataCenter.luxuryCashData:checkShopRedDot();
			self.itemTxt.text = numberTrans(DataCenter.packData:getItemCountById(self.costId));
			break;
		end
	end
end

function Lua_LuxuryCashPanel:onAddBtnClick()
	if self.costId == nil then return end
	DataCenter.packUtil:openPathWindow(self.costId, false);
end

function Lua_LuxuryCashPanel:onGoBtnClick()
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end
	local param = {actVo = self.actVo, index = 1};
	Lua_WindowUtil.openWin("Lua_LuxuryCashShopWindow", Lua_WinParam.ACTION_OPEN, param);
end



















Lua_LuxuryCashItem = class("Lua_LuxuryCashItem", Lua_ItemIconCell);

function Lua_LuxuryCashItem:buildShowItem()
	self:superCall("buildShowItem")
	self.itemIcon:setScale(0.95, 0.95, 0.95);
end

function Lua_LuxuryCashItem:setData(data)
	self.data = Lua_ToolVo.createToolVo(data.key, data.value);
	self.itemIcon:setData(self.data);
	self.itemIcon:showText(4, nil);
end