-- 豪礼兑换-任务界面
Lua_LuxuryCashTaskWindow = class("Lua_LuxuryCashTaskWindow", Lua_Window);
function Lua_LuxuryCashTaskWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_LuxuryCashTaskWindow";
	self.skinName = "LuxuryCashTaskWindow_skin";
	self.layer = MainWinDefine.NORMAL_WINDOW_LAYER;
	self.isFullSizeWin = true;
	self.showWinBg = true;
	self.winBgPath = "/images/cash_reward/xui_dhhd1_bg_3.jpg";
	self:addPreLoadRes(self.winBgPath, Lua_LoadFileType.IMG);
end

function Lua_LuxuryCashTaskWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.taskList = Lua_DragList.new(self.content, 687, 950 + self.adaptHeight, Lua_AxisDefine.Vertical);
	self.taskList:setListCell(Lua_LuxuryCashTaskItem, 687, 168, 1);
	self.taskList:setCellSpacing(0, 5);
	self.taskList:setLocation(-7.5, -116, 0);
	self.taskList.tweenId = 2002;

	self:addLuaEvent("UPDATE_LUXURYCASH_TASK_INFO", self, self.updateLuxuryCashTaskInfo);
	self:addLuaEvent("UPDATE_LUXURYCASH_INFO", self, self.onUpdateCost);
	self:addLuaEvent("ON_LUXURYCASH_DRAWREWARD_CLICK", self, self.onLuxuryCashDrawRewardClick);
end

function Lua_LuxuryCashTaskWindow:showWin(param)
	self:superCall("showWin", param);

	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sGetLuxuryCashTaskInfo(param.actIdx);

	local curActIdx = DataCenter.luxuryCashData.actIdx;
	if curActIdx == nil or curActIdx ~= param.actIdx then
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(param.actIdx);
	else
		self:onUpdateCost();
	end
end

function Lua_LuxuryCashTaskWindow:onScreenResize()
	self:superCall("onScreenResize");
	self.taskList:setContentRect(687, 950 + self.adaptHeight);
end

function Lua_LuxuryCashTaskWindow:onUpdateCost()
	local itemVo = DataCenter.luxuryCashData:getCostItemVo(Lua_LuxuryCashData.norType);
	self.icon:setAltas(itemVo.iconAtlas, itemVo.iconSprite);
	self.itemTxt.text = numberTrans(DataCenter.packData:getItemCountById(itemVo.id));

	self.time = DataCenter.luxuryCashData.closeTime - DataCenter.roleData.serverTime;
	self:startTimer();
	self:updateLeftTime();
end

function Lua_LuxuryCashTaskWindow:startTimer()
	if self.countDown ~= true then
		self.countDown = true;
		self:addTimer("Lua_LuxuryCashTaskWindow", 1, self, self.updateLeftTime);
	end
end

function Lua_LuxuryCashTaskWindow:updateLeftTime()
	self.time = self.time - 1;
	self.timeTxt.text = LanUtil.formateStr(203009, Lua_TimeTransform.transSeconds16(self.time, true, true, true, false));
	if self.time <= 0 then
		self.timeTxt:setLangValue(795001);
		self:stopTimer();
	end
end

function Lua_LuxuryCashTaskWindow:stopTimer()
	if self.countDown == true then
		self.countDown = false;
		self:removeTimer(self, self.updateLeftTime);
	end
end

function Lua_LuxuryCashTaskWindow:updateLuxuryCashTaskInfo(msg)
	if msg == nil then
		return;
	end

	-- 可领取＞未达成＞已领取，info1：0未完成 1可领取 2已领取
	table.sort( msg.task_info,
		function(a, b)
			if a.info1 == b.info1 then
				return a.task_id < b.task_id;
			else
				local aWeight = (a.info1 + 1) % 3;
				local bWeight = (b.info1 + 1) % 3;
				return aWeight > bWeight;
			end
		end
	);
	self.taskList:setArrayData(msg.task_info);
end

function Lua_LuxuryCashTaskWindow:onLuxuryCashDrawRewardClick(data)
	self:onLuxuryCashTaskOperateFun(1, data);
end

function Lua_LuxuryCashTaskWindow:onLuxuryCashTaskOperateFun(operateType, param)
	if DataCenter.backstageActData:checkActIsEnd(self.showData) then
		self:hide();
		return;
	end
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sDrawGetLuxuryCashTaskReward(self.showData.actIdx, tonumber(param.info4), param.task_id);
end

function Lua_LuxuryCashTaskWindow:dispose()
	if self.taskList ~= nil then
		self.taskList:dispose();
		self.taskList = nil;
	end
	self:stopTimer();
	self:superCall("dispose");
end

function Lua_LuxuryCashTaskWindow:onBackBtnClick(go)
	if not Lua_WindowUtil.isOpenWin("Lua_LuxuryCashShopWindow") then
		local param = {actVo = self.showData, index = 1};
		Lua_WindowUtil.openWin("Lua_LuxuryCashShopWindow", Lua_WinParam.ACTION_OPEN, param);
	end
	self:hide();
end























Lua_LuxuryCashTaskItem = class("Lua_LuxuryCashTaskItem", Lua_ListCell);
function Lua_LuxuryCashTaskItem:ctor(p)
	self:superCall("ctor", p, "LuxuryCashTaskItem_skin");
end

function Lua_LuxuryCashTaskItem:buildShowItem()
	self:superCall("buildShowItem");

	self.rewardList = Lua_DragList.new(self.content, 360, 87, Lua_AxisDefine.Horizontal);
	self.rewardList:setListCell(Lua_LuxuryCashTaskRewardItem, 87, 87, 1);
	self.rewardList:setCellSpacing(9, 0);
	self.rewardList:setRectPivot(3);
	self.rewardList:setLocation(-280, -16, 0);
	self:setSubList(self.rewardList);

	self.buyBtn:addClickFuntion(self, self.onDrawRewardClick);
end

function Lua_LuxuryCashTaskItem:setData(data)
	self._data = data;
	if self._data == nil then
		return;
	end

	self.titleTxt.text = data.dec;
	self.rewardList.canDrag = #data.award > 3;
	self.rewardList:setArrayData(data.award);
	self.progressBar:setMaxValue(data.info2);
	self.progressBar:setValue(data.info3);
	self.progressBar:setText(data.info3 .. "/" .. data.info2);

	if not DataCenter.funcOpenData:isOpened(data.func_id) then
		self.buyBtn:setLangValue(795010);
	else
		if data.info1 == 0 then -- 未完成
			self.buyBtn:setLangValue(795011);
			self.buyBtn:setStyle(3);
		elseif data.info1 == 1 then -- 可领取
			self.buyBtn:setLangValue(795012);
			self.buyBtn:setStyle(1);
		end -- 已领取
	end
	self.progressBar:setEnable(data.info1 ~= 2);
	self.buyBtn:setEnable(data.info1 ~= 2);
	self.tipsTxt:setEnable(tonumber(data.info4) == 1);
	self.stateIcon:setEnable(data.info1 == 2);
	self:showRedPoint(data.info1 == 1);
end

function Lua_LuxuryCashTaskItem:showRedPoint(bShow)
	if bShow == true then
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content);
			self.redPoint:createRedPointEff();
			self.redPoint:setLocation(297, 5, 0);
		end
		self.redPoint:updateRedPointShow(true);
	else
		if self.redPoint ~= nil then
			self.redPoint:updateRedPointShow(false);
		end
	end
end

function Lua_LuxuryCashTaskItem:onDrawRewardClick(go)
	if self._data then
		if DataCenter.funcOpenData:isOpened(self._data.func_id, true) then
			if self._data.info1 == 0 then
				DataCenter.funcPathUtil:gotoFunction(self._data.func_path_id, true)
			else
				Lua_MsgUtil.callFunc("ON_LUXURYCASH_DRAWREWARD_CLICK", self._data);
			end
		end
	end
end

function Lua_LuxuryCashTaskItem:dispose()
	if self.rewardList ~= nil then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self._data = nil;
	self:superCall("dispose");
end










Lua_LuxuryCashTaskRewardItem = class("Lua_LuxuryCashTaskRewardItem", Lua_ItemIconCell);
function Lua_LuxuryCashTaskRewardItem:buildShowItem()
	self:superCall("buildShowItem");
	self.itemIcon:setScale(0.7, 0.7);
end