-- 干脆面碰碰挑战界面
Lua_MergeBlockChallengeWindow = class("Lua_MergeBlockChallengeWindow", Lua_Window);
function Lua_MergeBlockChallengeWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_MergeBlockChallengeWindow";
	self.skinName = "MergeBlockChallengeWindow_skin";
	self.isFullSizeWin = true;
	self.showWinBg = true;
	self.showBackBtn = true;
	self.winBgPath = "/images/activities/xui_ppgcm_bg_2.jpg";
	self.winBgPosY = -180;
end

function Lua_MergeBlockChallengeWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.winBg.anchorValue = 7;
	self.winBg.pivotValue = 7;

	self.isFirstIn = true;
	self.mergeMaxNum = 11;
	self.orderNum = 4; -- 定义阶数
	self.totalNum = self.orderNum * self.orderNum;
	self.startPosX = -2;
	self.startPosY = 699;
	self.distance = 153;
	self.preScore = 0;
	self.recordMergeScore = 0;
	self.isFail = false;
	self.posNumDic = {}; -- 位置-数字
	self.showItemDic = {}; -- 位置-方块item
	self.endTweenArr = Array();

	self.slideBtn:addMousDownFuntion(self, self.onMaskDownClick);
	self.slideBtn:addMouseUpFuntion(self, self.onMaskUpClick);
	self.ruleBtn:addMouseUpFuntion(self, self.onRuleClick);

	self:addLuaEvent("UPDATE_MERGEBLOCK_INFO", self, self.showMaxPlayerInfo);
	self:addLuaEvent("UPDATE_MERGEBLOCK_CHALLENGE", self, self.updateShowInfo);
	self:addLuaEvent("ON_MERGEBLOCK_ENDTWEEN", self, self.updateTweenShowInfo);
end

function Lua_MergeBlockChallengeWindow:showWin(param)
	self:superCall("showWin", param);

	self:showMaxPlayerInfo();
	self:initGame();
	local vo = DataCenter.mergeBlockData.saveInfo;
	if vo.game_info and #vo.game_info <= 0 then
		self.isFirstIn = false;
		self:sComputeScore(1);
	end
end

function Lua_MergeBlockChallengeWindow:showMaxPlayerInfo()
	self.historyTxt.text = LanUtil.formateStr(773007, self.showData.levelInfo.top_score);

	local topPlayerInfo = self.showData.levelInfo.top_player;
	local name = topPlayerInfo.player_show.name;
	if strIsNullOrEmpty(name) then
		name = LanUtil.getValue(773008);
	end
	self.nameTxt.text = LanUtil.formateStr(773009, name);
	self.maxScoreTxt.text = LanUtil.formateStr(773010, topPlayerInfo.value);
end

function Lua_MergeBlockChallengeWindow:initGame()
	DataCenter.mergeBlockData.isMergeBlockTween = true;

	local vo = DataCenter.mergeBlockData.saveInfo;
	local line, column;
	for i = 1, self.totalNum do
		line, column = self:getLineAndColumnByIdx(i);
		self.posNumDic[i] = tonumber(vo.game_info and vo.game_info[i] or 0);

		item = Lua_MergeBlockChallengeItem.new(self.content);
		item.contentName = "blockItem_" .. i;
		item:setDefaultInfo(self.showData.actVo.theme, i, line, column);
		item:setRectAnchor(7);
		item:setEnable(false);
		self.showItemDic[i] = item;
		if self.posNumDic[i] ~= 0 then
			self:createNumItem(i);
		end
	end
	self.slideBtn.siblingIndex = self.showItemDic[self.totalNum].siblingIndex + 1;
	self.ruleBtn.siblingIndex = self.slideBtn.siblingIndex + 1;
	self.bottomBg.siblingIndex = self.ruleBtn.siblingIndex + 1;
	self.backBtn.siblingIndex = self.bottomBg.siblingIndex + 1;
	self:setCurScore(vo.last_score or 0);
end

function Lua_MergeBlockChallengeWindow:createNumItem(idx)
	self.showItemDic[idx].isAnimate = true;
	self.showItemDic[idx]:setScale(0.1, 0.1, 1);
	self:setShowBlockItem(idx, true);
	TweenUtil.getInstance():addTweenTargetScale(self.showItemDic[idx].mainContent,
		0.1, 0.05, 1, 1, 1, self, self.stopScaleTween, 0, true, idx);
end

function Lua_MergeBlockChallengeWindow:setShowBlockItem(idx, bShow)
	local line, column = self:getLineAndColumnByIdx(idx);
	local posX = self.startPosX + (column - (self.orderNum + 1) / 2) * self.distance;
	local posY = self.startPosY - (line - 1) * self.distance;
	self.showItemDic[idx]:setLocation(posX, posY, 0);
	self.showItemDic[idx]:setData(self.posNumDic[idx]);
	self.showItemDic[idx]:setEnable(bShow);
end

function Lua_MergeBlockChallengeWindow:stopScaleTween(idx)
	self.showItemDic[idx].isAnimate = false;
	TweenUtil.getInstance():stop(self.showItemDic[idx].mainContent);
end

function Lua_MergeBlockChallengeWindow:setCurScore(score)
	self.preScore = score;
	self.myScoreTxt.text = LanUtil.formateStr(773011, score);
end

function Lua_MergeBlockChallengeWindow:sComputeScore(challengeType, moveType)
	local blockArr = Array();
	local gameArr = Array();
	for i = 1, self.totalNum do
		if self.posNumDic[i] == 0 then
			blockArr:add(i);
		end
		gameArr:add(self.posNumDic[i]);
	end
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sComputeMergeBlockScore(
		self.showData.actVo.actIdx, self.showData.levelInfo.diffType, self.recordMergeScore, blockArr, gameArr, challengeType, moveType or 0);
	self.recordMergeScore = 0;
end

function Lua_MergeBlockChallengeWindow:updateShowInfo(msg)
	if msg == nil then
		return;
	end
	self.serverInfo = msg;

	if not self.isFirstIn or self.isCanSlide then
		self.isFirstIn = true;
		self:updateTweenShowInfo({operateType  = 2});
	end
end

function Lua_MergeBlockChallengeWindow:updateTweenShowInfo(param)
	if param.operateType == 1 then -- 合并显示
		self:setShowBlockItem(param.mergeIdx, true);
	elseif param.operateType == 2 then -- 结果显示
		if param.mergeIdx ~= nil then
			self.endTweenArr:add(param);
		end
		if self.isCanSlide and self.serverInfo then
			self:reSetChallengeShow();
			local len = self.endTweenArr:getSize();
			local info;
			for i = 1, len do
				info = self.endTweenArr:shift();
				self.showItemDic[info.mergeIdx].isAnimate = true;
				self.showItemDic[info.mergeIdx]:setScale(1.1, 1.1, 1);
				TweenUtil.getInstance():addTweenTargetScale(self.showItemDic[info.mergeIdx].mainContent,
					0.1, 0.05, 1, 1, 1, self, self.stopScaleTween, 0, true, info.mergeIdx);
			end
			self:setCurScore(self.serverInfo.latest_score);

			local list = self.serverInfo.new_block;
			for i = 1, #list do
				self.posNumDic[tonumber(list[i].key)] = tonumber(list[i].value);
				self:createNumItem(tonumber(list[i].key));
			end
			self.serverInfo = nil;

			if self.resultState == 0 then -- 游戏结束
				self:showChallengeResult();
			end
		end
	end
end

-- 重置挑战显示
function Lua_MergeBlockChallengeWindow:reSetChallengeShow()
	for i = 1, self.totalNum do
		TweenUtil.getInstance():stop(self.showItemDic[i].mainContent);
		self.showItemDic[i].isAnimate = false;
		self.showItemDic[i]:stopTimer();
		self:setShowBlockItem(i, self.posNumDic[i] ~= 0);
	end
end

function Lua_MergeBlockChallengeWindow:showChallengeResult()
	self.isFail = true;
	self:sComputeScore(3);
	Lua_WindowUtil.openWin("Lua_MergeBlockResultWindow", Lua_WinParam.ACTION_OPEN, {levelInfo = self.showData.levelInfo, result = self.preScore});
end

function Lua_MergeBlockChallengeWindow:onMaskDownClick(go)
	self.startMouseX = CommonUtil.mousePositionX;
	self.startMouseY = CommonUtil.mousePositionY;
	Lua_CommonUtil.delayCall(DataCenter.mergeBlockData.maxSlidingTime, self, self.updatePointTimer);
end

function Lua_MergeBlockChallengeWindow:updatePointTimer()
	self.isAutoSlide = self:onMaskUpClick();
end

function Lua_MergeBlockChallengeWindow:onMaskUpClick(go)
	if DataCenter.backstageActData:checkActIsEnd(self.showData.actVo) then
		self.isFail = true;
		self:hide();
		return;
	end

	Lua_CommonUtil.removeTimer(self, self.updatePointTimer);
	if self.isAutoSlide then
		self.isAutoSlide = false;
		return false;
	end

	if self.isFail or not self.isCanSlide then
		-- print(LanUtil.formateStr("失败：{1}，滑动：{2}", self.isFail, self.isCanSlide));
		return false; -- 已经失败或还有块播放动画不进行滑动计算
	end

	local minSlidingDistance = DataCenter.mergeBlockData.minSlidingDistance;
	local maxSlidingAngle = DataCenter.mergeBlockData.maxSlidingAngle;
	local verticalSquareCos = DataCenter.mergeBlockData.verticalSquareCos;
	local horizontalSquareCos = DataCenter.mergeBlockData.horizontalSquareCos;
	local a = CommonUtil.mousePositionX - self.startMouseX;
	local b = CommonUtil.mousePositionY - self.startMouseY;
	local squareCosB = a * a / (a * a + b * b);
	if squareCosB >= 0 and squareCosB <= verticalSquareCos then
		if b > 0 and b >= minSlidingDistance then -- 向上
			self:onOperateFun(1);
			return true;
		elseif b < 0 and -b >= minSlidingDistance then -- 向下
			self:onOperateFun(2);
			return true;
		end
	elseif squareCosB >= horizontalSquareCos and squareCosB <= 1 then
		if a < 0 and -a >= minSlidingDistance then -- 向左
			self:onOperateFun(3);
			return true;
		elseif a > 0 and a >= minSlidingDistance then -- 向右
			self:onOperateFun(4);
			return true;
		end
	end
end

-- 1上 2下 3左 4右
function Lua_MergeBlockChallengeWindow:onOperateFun(moveType)
	self:reSetChallengeShow();
	self:showResultToConsole("<color=#FFFF00>滑动</color>");

	local startIdx, endIdx, step, moveIdx, compareLine, compareColumn, moveLine, moveColumn, isMove, isMerger;
	if moveType == 1 or moveType == 3 then
		startIdx = 1;
		endIdx = self.totalNum;
		step = 1;
	else
		startIdx = self.totalNum;
		endIdx = 1;
		step = -1;
	end
	for i = startIdx, endIdx, step do
		moveIdx = self:getTheFirstNotZeroNum(i, moveType);
		if moveIdx ~= 0 then
			compareLine, compareColumn = self:getLineAndColumnByIdx(i);
			moveLine, moveColumn = self:getLineAndColumnByIdx(moveIdx);
			print(i, moveIdx, self.posNumDic[i], self.posNumDic[moveIdx])
			if self.posNumDic[i] == self.posNumDic[moveIdx] then -- 相同合并
				if self.posNumDic[i] < self.mergeMaxNum then -- 小于最大合并数
					isMerger = true;
					local num = tonumber(self.posNumDic[i]) + 1;
					local score = DataCenter.mergeBlockData.ruleDic[num].int2;
					self.recordMergeScore = self.recordMergeScore + score;
					self.posNumDic[i] = num; -- 相同+1
					self.posNumDic[moveIdx] = 0; -- 标记空位
					self:setTweenInfo(moveIdx, moveType, compareLine, compareColumn, moveLine, moveColumn, i);
				end
			else -- 不同移动
				if self.posNumDic[i] == 0 then
					isMove = true;
					self.posNumDic[i] = tonumber(self.posNumDic[moveIdx]); -- 覆盖值
					self.posNumDic[moveIdx] = 0; -- 标记空位
					self:setTweenInfo(moveIdx, moveType, compareLine, compareColumn, moveLine, moveColumn, nil);

					moveIdx = self:getTheFirstNotZeroNum(i, moveType);
					if (moveIdx ~= 0) and (self.posNumDic[i] == self.posNumDic[moveIdx]) and (self.posNumDic[i] < self.mergeMaxNum) then
						isMerger = true;
						local num = tonumber(self.posNumDic[i]) + 1;
						local score = DataCenter.mergeBlockData.ruleDic[num].int2;
						self.recordMergeScore = self.recordMergeScore + score;
						self.posNumDic[i] = num; -- 相同+1
						self.posNumDic[moveIdx] = 0; -- 标记空位
						moveLine, moveColumn = self:getLineAndColumnByIdx(moveIdx);
						self:setTweenInfo(moveIdx, moveType, compareLine, compareColumn, moveLine, moveColumn, i);
						-- print("移动合并", i, moveIdx)
					end
				else
					local nextIdx = self.showItemDic[i]:getTheItemNextIdx(moveType);
					if nextIdx ~= moveIdx then
						isMove = true;
						self.posNumDic[nextIdx] = tonumber(self.posNumDic[moveIdx]); -- 覆盖值
						self.posNumDic[moveIdx] = 0; -- 标记空位
						compareLine, compareColumn = self:getLineAndColumnByIdx(nextIdx);
						self:setTweenInfo(moveIdx, moveType, compareLine, compareColumn, moveLine, moveColumn, nil);
					end
					-- print("nextIdx", i, nextIdx, moveIdx)
				end
			end
		end
	end
	self:resultFun(moveType, isMove, isMerger);
	self:showResultToConsole("结果，操作类型："..moveType, isMove, isMerger, self.resultState);
end

function Lua_MergeBlockChallengeWindow:setTweenInfo(moveIdx, moveType, compareLine, compareColumn, moveLine, moveColumn, mergeIdx)
	local moveNum;
	if moveType == 3 or moveType == 4 then
		moveNum = Mathf.Abs(compareColumn - moveColumn);
	else
		moveNum = Mathf.Abs(compareLine - moveLine);
	end
	self.showItemDic[moveIdx]:setItemTweenInfo(moveType, moveNum, mergeIdx);
	-- print(moveIdx, moveNum, mergeIdx);
end

function Lua_MergeBlockChallengeWindow:getTheFirstNotZeroNum(idx, moveType)
	local nextIdx = self.showItemDic[idx]:getTheItemNextIdx(moveType);
	if nextIdx == 0 then
		return 0;
	end
	if self.posNumDic[nextIdx] == 0 then
		nextIdx = self:getTheFirstNotZeroNum(nextIdx, moveType);
	end
	return nextIdx;
end

function Lua_MergeBlockChallengeWindow:resultFun(moveType, isMove, isMerger)
	if not isMove and not isMerger then
		return;
	end

	local startIdx = self.challengeBg.siblingIndex;
	for i = 1, self.totalNum do
		if moveType == 1 then -- 上
			self.showItemDic[i]:setSiblingIndex(startIdx + 1 + self.totalNum - i);
		elseif moveType == 2 then -- 下
			self.showItemDic[i]:setSiblingIndex(startIdx + i);
		elseif moveType == 3 then -- 左
			self.showItemDic[i]:setSiblingIndex(startIdx + 1 + self.totalNum - i);
		else -- 右
			self.showItemDic[i]:setSiblingIndex(startIdx + i);
		end
		self.showItemDic[i]:playBlockTween();
	end
	self:sComputeScore(2, moveType);
end

function Lua_MergeBlockChallengeWindow:showResultToConsole(titleDesStr)
	print(titleDesStr);
	local str = "";
	for i = 1, self.totalNum do
		str = str .. "__" .. self.posNumDic[i] .. "__ ";
		if i % self.orderNum == 0 then
			print(str);
			str = "";
		end
	end
end

function Lua_MergeBlockChallengeWindow:onRuleClick(go)
	Lua_WindowUtil.openWin("Lua_MergeBlockRuleWindow", Lua_WinParam.ACTION_OPEN, self.showData.actVo);
end

function Lua_MergeBlockChallengeWindow:onBackBtnClick(go)
	if not self.isOpen then
		local tipsData = {
			owner = self,
			des = LanUtil.getValue(773025),
			btnType = 1,
			cancelLabel = LanUtil.getValue(773026),
			sureLabel = LanUtil.getValue(773027),
			cancelFun = self.showChallengeResult,
			sureFun = self.hide
		};
		Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
	end
end

-- 检查是否还有块在播放动画
function Lua_MergeBlockChallengeWindow.Get:isCanSlide()
	for i = 1, self.totalNum do
		if self.showItemDic[i].isAnimate then
			return false;
		end
	end
	return true;
end

-- 检查是否能继续游戏
function Lua_MergeBlockChallengeWindow.Get:resultState()
	if self.isSpace then -- 有空位继续游戏
		return 1;
	else
		local state = 0; -- 结束
		for i = 1, self.orderNum do
			for j = 1, self.orderNum - 1 do
				idx_1 = self:getIdxByLineAndColumn(i, j);
				idx_2 = self:getIdxByLineAndColumn(i, j + 1);
				idx_3 = self:getIdxByLineAndColumn(j, i);
				idx_4 = self:getIdxByLineAndColumn(j + 1, i);
				if (self.posNumDic[idx_1] == self.posNumDic[idx_2] and self.posNumDic[idx_1] < self.mergeMaxNum)
					or (self.posNumDic[idx_3] == self.posNumDic[idx_4] and self.posNumDic[idx_3] < self.mergeMaxNum) then
					state = 1; -- 存在左右相等或上下相等继续游戏
					break;
				end
			end
		end
		return state;
	end
end

-- 检查是否还有空位
function Lua_MergeBlockChallengeWindow.Get:isSpace()
	for i = 1, self.totalNum do
		if self.posNumDic[i] == 0 then
			return true;
		end
	end
	return false;
end

-- 根据行/列返回索引
function Lua_MergeBlockChallengeWindow:getIdxByLineAndColumn(lineIdx, columnIdx)
	return (lineIdx - 1) * self.orderNum + columnIdx;
end

-- 根据索引返回行/列
function Lua_MergeBlockChallengeWindow:getLineAndColumnByIdx(idx)
	local line = Mathf.Ceil(idx / self.orderNum);
	local column = idx - Mathf.Floor((idx - 1) / self.orderNum) * self.orderNum;
	return line, column;
end

function Lua_MergeBlockChallengeWindow:dispose()
	if not self.isFail then
		OperateAlert.showLangValue(773028);
	end
	if self.showItemDic ~= nil then
		for i = 1, self.totalNum do
			TweenUtil.getInstance():stop(self.showItemDic[i].mainContent);
			self.showItemDic[i]:dispose();
			self.showItemDic[i] = nil;
		end
		self.showItemDic = nil;
	end
	if self.endTweenArr ~= nil then
		self.endTweenArr:dispose();
		self.endTweenArr:clear();
		self.endTweenArr = nil;
	end
	self.posNumDic = nil;
	self.serverInfo = nil;
	DataCenter.mergeBlockData.isMergeBlockTween = false;
	Lua_CommonUtil.removeTimer(self, self.updatePointTimer);
	Lua_MsgUtil.callFunc("ON_MERGEBLOCK_REFRESH_CLICK");
	self:superCall("dispose");
end










Lua_MergeBlockChallengeItem = class("Lua_MergeBlockChallengeItem", Lua_DisplayObject);
function Lua_MergeBlockChallengeItem:ctor(p)
	self:superCall("ctor", p, "MergeBlockChallengeItem_skin");
end

function Lua_MergeBlockChallengeItem:buildShowItem()
	self:superCall("buildShowItem");

	self.orderNum = 4;
	self.distance = 153;
	self.maxDistance = (self.orderNum - 1) * self.distance;
	self.changeWidth = 50;
	self.upIdx = 0;
	self.downIdx = 0;
	self.leftIdx = 0;
	self.rightIdx = 0;
	self:initTweenInfo();
end

function Lua_MergeBlockChallengeItem:initTweenInfo()
	self.moveType = nil;
	self.moveNum = 0;
	self.mergeIdx = nil;
end

function Lua_MergeBlockChallengeItem:setDefaultInfo(theme, idx, lineIdx, columnIdx)
	self.theme = theme;
	self.upIdx = lineIdx <= 1 and 0 or idx - self.orderNum;
	self.downIdx = lineIdx >= self.orderNum and 0 or idx + self.orderNum;
	self.leftIdx = columnIdx <= 1 and 0 or idx - 1;
	self.rightIdx = columnIdx >= self.orderNum and 0 or idx + 1;
end

function Lua_MergeBlockChallengeItem:getTheItemNextIdx(moveType)
	if moveType == 1 then
		return self.downIdx;
	elseif moveType == 2 then
		return self.upIdx;
	elseif moveType == 3 then
		return self.rightIdx;
	else
		return self.leftIdx;
	end
end

function Lua_MergeBlockChallengeItem:setData(data)
	if data == nil then
		return;
	end
	self.itemBg:setAltas("merge_block_atlas_1", "xui_ppgcm_icon_bg_" .. data);
	self.icon:setAltas("merge_block_atlas_1", "xui_ppgcm_icom_" .. self.theme * 10000 + data);
	self.icon:resetSize();
	self.numTxt.text = data;
end

function Lua_MergeBlockChallengeItem:setItemTweenInfo(moveType, moveNum, mergeIdx)
	self.moveType = self.moveType or moveType;
	self.moveNum = self.moveNum + moveNum;
	self.mergeIdx = self.mergeIdx or mergeIdx;
end

function Lua_MergeBlockChallengeItem:playBlockTween()
	if self.moveType ~= nil then
		self.isAnimate = true;
		self:startTimer();
	end
end

function Lua_MergeBlockChallengeItem:startTimer()
	if self.countDown ~= true then
		self.countDown = true;

		self.remainTime = getTime();
		self.movePos = 0;
		self.targetDistance = self.moveNum * self.distance;
		self.targetDistance = self.targetDistance > self.maxDistance and self.maxDistance or self.targetDistance;
		self.isCanMerge = self.mergeIdx ~= nil;
		Lua_CommonUtil.addTimer("Lua_MergeBlockChallengeItem", 0.01, self, self.updateTimer, nil, -1, true);
	end
end

function Lua_MergeBlockChallengeItem:updateTimer()
	local newTime = getTime();
	local dt = newTime - self.remainTime;
	self.remainTime = newTime;

	self.movePos = self.movePos + dt / DataCenter.mergeBlockData.slideTime * self.targetDistance;
	self.movePos = self.movePos > self.targetDistance and self.targetDistance or self.movePos;
	local targetX, targetY = self:getTheTarget(self.moveType, self.movePos);
	LuaUtil.setObjPos(self.mainContent, targetX, targetY, 0);

	if self.movePos >= self.targetDistance - self.changeWidth and self.isCanMerge then
		self.isCanMerge = false;
		Lua_MsgUtil.callFunc("ON_MERGEBLOCK_ENDTWEEN", {operateType = 1, mergeIdx = self.mergeIdx});
	end
	if self.movePos >= self.targetDistance then
		self.isAnimate = false;
		self:stopTimer();
		Lua_MsgUtil.callFunc("ON_MERGEBLOCK_ENDTWEEN", {operateType = 2, mergeIdx = self.mergeIdx});
		self:initTweenInfo();
	end
end

function Lua_MergeBlockChallengeItem:getTheTarget(moveType, moveDistance)
	local targetX, targetY;
	if moveType == 1 then -- 上
		targetX = self.x
		targetY = self.y + moveDistance;
	elseif moveType == 2 then -- 下
		targetX = self.x
		targetY = self.y - moveDistance;
	elseif moveType == 3 then -- 左
		targetX = self.x - moveDistance;
		targetY = self.y
	else -- 右
		targetX = self.x + moveDistance;
		targetY = self.y
	end
	return targetX, targetY;
end

function Lua_MergeBlockChallengeItem:stopTimer()
	if self.countDown == true then
		self.countDown = false;
		Lua_CommonUtil.removeTimer(self, self.updateTimer, true);
	end
end

function Lua_MergeBlockChallengeItem.Get:isAnimate()
	return self._isAnimate;
end

function Lua_MergeBlockChallengeItem.Set:isAnimate(val)
	if val ~= self._isAnimate then
		self._isAnimate = val;
	end
end

function Lua_MergeBlockChallengeItem:dispose()
	self._isAnimate = nil;
	self:initTweenInfo();
	self:stopTimer();
	self:superCall("dispose");
end