-- 皮肤直购窗口
Lua_SkinPurchaseSureShopWindow = class("Lua_SkinPurchaseSureShopWindow", Lua_Window);
function Lua_SkinPurchaseSureShopWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_SkinPurchaseSureShopWindow";
	self.skinName = "SkinPurchaseSureShopWindow_skin";
	self.isFullSizeWin = true;
	self.showBlackMask = true;
	self.isClickClose = true;
end

function Lua_SkinPurchaseSureShopWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.listHeight = 124;
	self.itemWeight = 128;
	self.distanceX = 2;
	local weight = 4.5 * self.itemWeight + 3 * self.distanceX;
	self.itemList = Lua_DragList.new(self.content, weight, self.listHeight, Lua_AxisDefine.Horizontal);
	self.itemList:setLocation(-7, -38);
	self.itemList:setListCell(Lua_SkinPurchaseSureShopItem, self.itemWeight, 124, 1);
	self.itemList:setCellSpacing(self.distanceX, 0);

	self.shopBtn:addClickFuntion(self, self.onClickShopBtn);

	self:addLuaEvent("UPDATE_SKINPURCHASE_INFO", self, self.updateRechargeInfo);
end

function Lua_SkinPurchaseSureShopWindow:showWin(param)
	self:superCall("showWin", param);

	self.msgData = param.msgData;
	self.activity_vo = param.activity_vo;

	self:updateInfo();
end

function Lua_SkinPurchaseSureShopWindow:dispose()
	self.msgData = nil;
	self.activity_vo = nil;
	if self.itemList ~= nil then
		self.itemList:dispose();
		self.itemList = nil;
	end
	
	self:superCall("dispose");
end

function Lua_SkinPurchaseSureShopWindow:updateRechargeInfo(msg)
	self.msgData = msg;
	self:updateInfo();
end

function Lua_SkinPurchaseSureShopWindow:updateInfo()
	self:updateItems();
	self:updateShopTxt();
	self:updateBtnInfo();
end

function Lua_SkinPurchaseSureShopWindow:updateItems()
	local itemList = self.msgData.recharge_reward.item_list;
	self.itemList:setArrayData(itemList);

	local weight;
	local count = itemList and #itemList or 0;
	if count > 4 then
		weight = 4.5 * self.itemWeight + 3 * self.distanceX;
	else
		weight = count * self.itemWeight + (count - 1) * self.distanceX;
	end

	self.itemList:setContentRect(weight, self.listHeight);
	self.itemList.canDrag = count > 4;
end

function Lua_SkinPurchaseSureShopWindow:updateShopTxt()
	local exchange_item = self.msgData.exchange_item;
	local exchange_id = tonumber(exchange_item.key);
	local exchange_num = tonumber(exchange_item.value);

	local str = spliceIconStrById(exchange_id);
	str = LanUtil.formateStr(788005, str, exchange_num);
	self.descTxt.text = str;
end

function Lua_SkinPurchaseSureShopWindow:updateBtnInfo()
	local shopBuyId = self.msgData.recharge_id;
	local rechargeInfo = Lua_RechargeConfigData.getRechargeConfig(shopBuyId);
	local money = rechargeInfo and rechargeInfo.money or 0;
	self.shopBtn.label = LanUtil.formateStr(788006, money);
end

function Lua_SkinPurchaseSureShopWindow:onClickShopBtn()
	if self.clickTime ~= nil and getTime() - self.clickTime < 0.5 then
		return;
	end
	self.clickTime = getTime();
	
	if DataCenter.backstageActData:checkActIsEnd(self.activity_vo) then
		return;
	end

	local shopBuyId = self.msgData.recharge_id;
	local rechargeInfo = Lua_RechargeConfigData.getRechargeConfig(shopBuyId);
	DataCenter.sdkInterface:PointOut(1, tonumber(rechargeInfo.money), tonumber(rechargeInfo.money_type),
				tonumber(rechargeInfo.id), rechargeInfo.gift_name, rechargeInfo.gold);

	self:hide();
end
















---------道具item-----------
Lua_SkinPurchaseSureShopItem = class("Lua_SkinPurchaseSureShopItem", Lua_ListCell);
function Lua_SkinPurchaseSureShopItem:ctor(p)
	self:superCall("ctor", p, "SkinPurchaseSureShopItem_skin");
end

function Lua_SkinPurchaseSureShopItem:buildShowItem()
	self:superCall("buildShowItem");

	self.itemIcon = Lua_ItemIcon.new(self.content);
	self.itemIcon:setScale(0.9, 0.9, 0.9);
	self.itemIcon:setLocation(7, -6);
	self.itemIcon:setSiblingIndex(0);
end

function Lua_SkinPurchaseSureShopItem:setData(data)
	self.tvo = nil;
    if data == nil then
		return;
	end

	local itemId = tonumber(data.id);
	local skinItemId = DataCenter.skinPurchaseData.skinItemId;
	if self.tvo == nil then
		self.tvo = Lua_ToolVo.createToolVo(itemId);
	end

	local bChange = DataCenter.skinPurchaseData.has_cloth and skinItemId == itemId;
	if bChange then
		local exchange_item = DataCenter.skinPurchaseData.exchange_item;
		local exchange_id = tonumber(exchange_item.key);
		local exchange_value = tonumber(exchange_item.value);
		self.tvo:setData(exchange_id, exchange_value);
	else
		self.tvo:fromServer(data);
	end
	self.changeSp:setEnable(bChange);
	self.itemIcon:setData(self.tvo);
end

function Lua_SkinPurchaseSureShopItem:onPressClick()
	if self.tvo == nil then
		return;
	end

	DataCenter.packUtil:showItemInfo(self.tvo);
end

function Lua_SkinPurchaseSureShopItem:dispose()
	self.tvo = nil;
	if self.itemIcon then
		self.itemIcon:dispose();
		self.itemIcon = nil;
	end
    self:superCall("dispose");
end