-- 异兽争胜 panel
Lua_StrangeBeastMatchPanel = class("Lua_StrangeBeastMatchPanel", Lua_DisplayObject);

function Lua_StrangeBeastMatchPanel:ctor(p, skinName)
	self:superCall("ctor", p, skinName);
end

function Lua_StrangeBeastMatchPanel:buildPanel()
	self:superCall("buildPanel");

	self:addLuaEvent("UPDATE_STRANGEBEASTMATCH_INFO", self, self.onUpdateInfo);
	self:addLuaEvent("UPDATE_STRANGEBEASTMATCH_TASK_INFO", self, self.onUpdateRedDot);

	self:addLuaEvent("GET_STRANGEBEASTMATCH_SOCRE_REWARD", self, self.onGetReward);
end

function Lua_StrangeBeastMatchPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.ruleTxt:setLangValue(786000);

	self.dragList = Lua_DragList.new(self.content, 720, 850 + self.adaptHeight, Lua_AxisDefine.Vertical);
	self.dragList:setListCell(Lua_StrangeBeastMatchItem, 720, 185, 1);
	self.dragList:setCellSpacing(0, -10);
	self.dragList:setLocation(0, -85, 0);

	self.taskRedPoint = Lua_RedPointIcon.new(self.content, Lua_RedPointList.StrangeBeastMatchTask);
	self.taskRedPoint:setRectAnchor(1);
	self.taskRedPoint:setLocation(325, -208, 0);

	self.tipBtn:addClickFuntion(self, self.onTipBtnClick);
	self.rankBtn:addClickFuntion(self, self.onRankBtnClick);
	self.taskBtn:addClickFuntion(self, self.onTaskBtnClick);
end

function Lua_StrangeBeastMatchPanel:dispose()
	if self.dragList ~= nil then
		self.dragList:dispose();
		self.dragList = nil;
	end
	if self.redPointIcon ~= nil then
		self.redPointIcon:dispose();
		self.redPointIcon = nil;
	end
	self:superCall("dispose");
end

function Lua_StrangeBeastMatchPanel:onScreenResize(adaptHeight)
	self:superCall("onScreenResize", adaptHeight);

	self.dragList:setContentRect(720, 850 + adaptHeight);
end

function Lua_StrangeBeastMatchPanel:setData(actVo)
	self.actVo = actVo;
	self:initTimer();

	-- 任务红点
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sStrangeBeastMatchTaskInfo(actVo.actIdx, 1);
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sStrangeBeastMatchTaskInfo(actVo.actIdx, 2);
end

function Lua_StrangeBeastMatchPanel:initTimer()
	if not self.isTimer then
		self.isTimer = true;
		self:onTimer();
		self:addTimer("onTimer", 1, self, self.onTimer);
	end
end

function Lua_StrangeBeastMatchPanel:onTimer()
	local time = self.actVo.residueTime;
	if time <= 0 then
		time = 0;
		self:endTimer();
	end
	self.timeTxt.text = LanUtil.formateStr(786001, Lua_TimeTransform.transSeconds18(time, true, true, true, true));
end

function Lua_StrangeBeastMatchPanel:endTimer()
	if self.isTimer then
		self.isTimer = false;
		self:removeTimer(self, self.onTimer);
	end
end

function Lua_StrangeBeastMatchPanel:onUpdateInfo(msg)
	self.socreTxt.text = LanUtil.formateStr(786002, numberTrans(msg.score));

	local list = msg.reward_list;
	table.sort(list,
		function(a, b)
			local aWeight = a.state == 2 and a.state * 1000 + a.id or a.id;
			local bWeight = b.state == 2 and b.state * 1000 + b.id or b.id;
			return aWeight < bWeight;
		end);
	self.dragList:setArrayData(list);
end

function Lua_StrangeBeastMatchPanel:onUpdateRedDot(msg)
	local state = false;
	local taskList = msg.task_info;
	for i = 1, #taskList do
		if taskList[i].info1 == 1 then
			state = true;
			break;
		end
	end
	
	if msg.type == 1 then
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.StrangeBeastMatchTaskDaily, state);
	else
		DataCenter.redPointData:updateRedPointState(Lua_RedPointList.StrangeBeastMatchTaskCumulate, state);
	end
end

function Lua_StrangeBeastMatchPanel:onTipBtnClick()
	local tipData = { des = self.actVo.help_des, posX = -185, posY = -460 };
	Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipData);
end

function Lua_StrangeBeastMatchPanel:onRankBtnClick()
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end
	Lua_WindowUtil.openWin("Lua_StrangeBeastMatchRankWindow", Lua_WinParam.ACTION_OPEN, self.actVo);
end

function Lua_StrangeBeastMatchPanel:onTaskBtnClick()
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end
	Lua_WindowUtil.openWin("Lua_StrangeBeastMatchTaskWindow", Lua_WinParam.ACTION_OPEN, self.actVo);
end

function Lua_StrangeBeastMatchPanel:onGetReward(id)
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sGetStrangeBeastMatchScoreReward(self.actVo.actIdx, id);
end

























Lua_StrangeBeastMatchItem = class("Lua_StrangeBeastMatchItem", Lua_ListCell);

function Lua_StrangeBeastMatchItem:ctor(p)
	self:superCall("ctor", p, "StrangeBeastMatchItem_skin");
end

function Lua_StrangeBeastMatchItem:buildShowItem()
	self:superCall("buildShowItem");

	self.rewardList = Lua_DragList.new(self.content, 410, 87, Lua_AxisDefine.Horizontal);
	self.rewardList:setListCell(Lua_StrangeBeastMatchRewardItem, 87, 87, 1);
	self.rewardList:setCellSpacing(5, 0);
	self.rewardList:setLocation(-99, -5, 0);
	self:setSubList(self.rewardList);

	self.getBtn:addClickFuntion(self, self.onGetBtnClick);
end

function Lua_StrangeBeastMatchItem:dispose()
	if self.rewardList ~= nil then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	self.data = nil;
	self:superCall("dispose");
end

function Lua_StrangeBeastMatchItem:setData(data)
	self.data = data;

	self.titleTxt.text = data.title;
	self.rewardList:setArrayData(data.item_list);

	if data.state == 0 then
		self.getBtn:setLangValue(786003);
		self.getBtn:setStyle(3);
	elseif data.state == 1 then
		self.getBtn:setLangValue(786004);
		self.getBtn:setStyle(1);
	end
	self.getBtn:setEnable(data.state ~= 2)
	self.stateIcon:setEnable(data.state == 2);

	self:updateRedDot(data.state == 1);
end

function Lua_StrangeBeastMatchItem:updateRedDot(bShow)
	if bShow and self.redPoint == nil then
		self.redPoint = Lua_RedPointIcon.new(self.content);
		self.redPoint:createRedPointEff();
		self.redPoint:setLocation(285, 20, 0);
		self.redPoint:updateRedPointShow(true);
	end
	if self.redPoint ~= nil then
		self.redPoint:updateRedPointShow(bShow);
	end
end

function Lua_StrangeBeastMatchItem:onGetBtnClick()
	if self.data.state == 0 then
		DataCenter.funcPathUtil:gotoFunction(1165, true);
	elseif self.data.state == 1 then
		Lua_MsgUtil.callFunc("GET_STRANGEBEASTMATCH_SOCRE_REWARD", self.data.id);
	end
end










Lua_StrangeBeastMatchRewardItem = class("Lua_StrangeBeastMatchRewardItem", Lua_ItemIconCell);

function Lua_StrangeBeastMatchRewardItem:buildShowItem()
	self:superCall("buildShowItem");

	self.itemIcon:setScale(0.7, 0.7);
end