-- 异兽争胜 排行panel
Lua_StrangeBeastMatchRankPanel = class("Lua_StrangeBeastMatchRankPanel", Lua_DisplayObject);

function Lua_StrangeBeastMatchRankPanel:ctor(p)
	self:superCall("ctor", p, "StrangeBeastMatchRankPanel_skin");
end

function Lua_StrangeBeastMatchRankPanel:buildPanel()
	self:superCall("buildPanel");
	self:addLuaEvent("UPDATE_STRANGEBEASTMATCH_RANK_INFO", self, self.onUpdateInfo);
end

function Lua_StrangeBeastMatchRankPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.avatarPosDic = {{0, -255, 0}, {-236, -270, 0}, {236, -270, 0}};
	self.avatarScaleDic = {0.9, 0.8, 0.8};

	self.rankArr = Array();

	self.roleInfoPanel = Lua_RoleInfoPanel.new(self.content);
	self.roleInfoPanel:setRectAnchor(1);

	self.dragList = Lua_DragList.new(self.content, 700, 520 + self.adaptHeight, Lua_AxisDefine.Vertical);
	self.dragList:setListCell(Lua_StrangeBeastMatchRankItem, 700, 125, 1);
	self.dragList:setLocation(0, -180, 0);
	self.dragList.callBackOwner = self;
	self.dragList.turnPageCallBack = self.onTurnPage;
end

function Lua_StrangeBeastMatchRankPanel:dispose()
	if self.roleInfoPanel ~= nil then
		self.roleInfoPanel:dispose();
		self.roleInfoPanel = nil;
	end
	if self.dragList ~= nil then
		self.dragList:dispose();
		self.dragList = nil;
	end
	if self.myHeadIcon ~= nil then
		self.myHeadIcon:dispose();
		self.myHeadIcon = nil;
	end
	if self.myRankNum ~= nil then
		self.myRankNum:dispose();
		self.myRankNum = nil;
	end
	self:superCall("dispose");
end

function Lua_StrangeBeastMatchRankPanel:onScreenResize(adaptHeight)
	self:superCall("onScreenResize", adaptHeight);

	self.dragList:setContentRect(700, 520 + adaptHeight);
end

function Lua_StrangeBeastMatchRankPanel:setData(actVo)
	self.actVo = actVo;
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sStrangeBeastMatchRankInfo(actVo.actIdx, 0);
end

function Lua_StrangeBeastMatchRankPanel:onTurnPage()
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sStrangeBeastMatchRankInfo(self.actVo.actIdx, 1);
end

function Lua_StrangeBeastMatchRankPanel:onUpdateInfo(msg)
	if msg.min_score <= 0 then
		self.tipTxt.text = "";
	else
		self.tipTxt.text = LanUtil.formateStr(786011, msg.min_score);
	end

	local rankInfo = msg.rank_list;
	for i = 1, #rankInfo do
		self.rankArr:add(rankInfo[i]);
	end

	local startIdx = 0;
	local topRankDic = {};
	for i = 1, 4 do
		rankInfo = self.rankArr[i];
		if rankInfo == nil then
			break;
		end
		if rankInfo.rank < 4 then
			topRankDic[rankInfo.rank] = rankInfo;
		else
			startIdx = i;
			break;
		end
	end
	if startIdx ~= 0 then
		self.dragList:setArrayData(self.rankArr, startIdx);
	end
	self.dragList.isPageEnd = msg.is_end;
	self:showNone(startIdx == 0);

	local valueTxt;
	local nameTxt;
	local avatarStr;
	local playerInfo;
	for i = 1, 3 do
		rankInfo = topRankDic[i];
		valueTxt = "valueTxt_" .. i;
		nameTxt = "nameTxt_" .. i;
		avatarStr = "avatarItem_" .. i;
		if self[avatarStr] == nil then
			self[avatarStr] = Lua_AvatarItem.new(self.content);
			self[avatarStr]:setLocation(self.avatarPosDic[i][1], self.avatarPosDic[i][2], self.avatarPosDic[i][3]);
			self[avatarStr]:setScale(self.avatarScaleDic[i], self.avatarScaleDic[i]);
			self[avatarStr]:setRectAnchor(1);
		end
		if rankInfo == nil then
			self[valueTxt].text = "";
			self[nameTxt]:setLangValue(786005);
			self[avatarStr]:init();
		else
			self[valueTxt].text = rankInfo.value;

			playerInfo = rankInfo.player_show;
			self[nameTxt].text = playerInfo.name;
			self[avatarStr]:setRoleId(playerInfo.id, playerInfo.server_flag);
			self[avatarStr]:setRoleDressUp(playerInfo.dress_info, true);
			self[avatarStr]:setRoleLevel2(playerInfo.level);
		end
	end

	rankInfo = msg.self_rank;
	if rankInfo == nil then
		self.myNameTxt.text = "";
		self.myScoreTxt.text = "";
		if self.myHeadIcon ~= nil then
			self.myHeadIcon:setEnable(false);
		end
		if self.myRankNum ~= nil then
			self.myRankNum:setEnable(false);
		end
		self.myNoRankIcon:setEnable(true);
	else
		if rankInfo.rank == 0 then
			if self.myRankNum ~= nil then
				self.myRankNum:setEnable(false);
			end
			self.myNoRankIcon:setEnable(true);
		else
			if self.myRankNum == nil then
				self.myRankNum = Lua_NumberImage.new(self.content);
				self.myRankNum:setRectAnchor(7);
				self.myRankNum:setLocation(-270.5, 186, 0);
				self.myRankNum.numOffsetX = -4;
				self.myRankNum.numAtlas = "number";
				self.myRankNum.preStr = "xui_phb_number_";
			end
			self.myRankNum.num = rankInfo.rank;
			self.myRankNum:setEnable(true);
			self.myNoRankIcon:setEnable(false);
		end
		playerInfo = rankInfo.player_show;
		self.myNameTxt.text = playerInfo.name;
		self.myScoreTxt.text = rankInfo.value;

		if self.myHeadIcon == nil then
			self.myHeadIcon = Lua_AvatarItem.new(self.content);
			self.myHeadIcon:setRectAnchor(7);
			self.myHeadIcon:setScale(0.7, 0.7, 0.7);
			self.myHeadIcon:setLocation(-147, 186, 0);
		end
		self.myHeadIcon:setRoleId(playerInfo.id, playerInfo.server_flag);
		self.myHeadIcon:setRoleDressUp(playerInfo.dress_info, true);
		self.myHeadIcon:setRoleLevel2(playerInfo.level);
	end
end

function Lua_StrangeBeastMatchRankPanel:showNone(state)
	if state then
		if self.noneIcon == nil then
			self.noneIcon = UICreate.createImage(self.content, "/images/common/xui_common_widget_029.png", 0, -174, 0, 0);
		end
		if self.noneText == nil then
			self.noneText = UICreate.createLabel(self.content, nil, 113, -140, 100, 40, 40);
			self.noneText.pivotValue = 4;
			self.noneText.alignmentValue = 4;
			self.noneText.horizontalOverflowValue = 1;
			self.noneText.fontName = Lua_MyUISettings.huakangFontName;
			self.noneText.colorStr = Lua_ColorDefine.transColorStr("E3D7DA");
			self.noneText:setLangValue(786006);
		end
	end

	if self.noneIcon ~= nil then
		self.noneIcon:setEnable(state);
	end
	if self.noneText ~= nil then
		self.noneText:setEnable(state);
	end
end




















Lua_StrangeBeastMatchRankItem = class("Lua_StrangeBeastMatchRankItem", Lua_ListCell);

function Lua_StrangeBeastMatchRankItem:ctor(p)
	self:superCall("ctor", p, "StrangeBeastMatchRankItem_skin");
end

function Lua_StrangeBeastMatchRankItem:buildShowItem()
	self:superCall("buildShowItem");

	self.avatarItem = Lua_AvatarItem.new(self.content);
	self.avatarItem:setScale(0.7, 0.7, 0.7);
	self.avatarItem:setLocation(-147, 0, 0);
end

function Lua_StrangeBeastMatchRankItem:dispose()
	if self.avatarItem ~= nil then
		self.avatarItem:dispose();
		self.avatarItem = nil
	end
	self.data = nil;
	self:superCall("dispose");
end

function Lua_StrangeBeastMatchRankItem:setData(data)
	self.data = data;

	self.rankTxt.text = data.rank;
	self.scoreTxt.text = data.value;

	local playerInfo = data.player_show;
	self.nameTxt.text = playerInfo.name;
	self.avatarItem:setRoleId(playerInfo.id, playerInfo.server_flag);
	self.avatarItem:setRoleDressUp(playerInfo.dress_info, true);
	self.avatarItem:setRoleLevel2(playerInfo.level);
end