-- 见缝插刀游玩界面
Lua_ThrowKnifeChallengeWindow = class("Lua_ThrowKnifeChallengeWindow", Lua_Window);
function Lua_ThrowKnifeChallengeWindow:ctor()
	self:superCall("ctor");

	self.windowName = "Lua_ThrowKnifeChallengeWindow";
	self.skinName = "ThrowKnifeChallengeWindow_skin";
	self.isFullSizeWin = true;
	self.showWinBg = true;
	self.showBackBtn = true;
	self.winBgPath = "/images/activities/xui_jfcd_bg_2.jpg";
end

function Lua_ThrowKnifeChallengeWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.upImg:setParent(self.turntableImg.mainContent);
	self.upImg:setLocation(0, 0, 0);
	self.mask:addMousDownFuntion(self, self.onThrowClick);

	self:addLuaEvent("UPDATE_THROWTKNIFE_CHALLENGE", self, self.updateChallengeInfo);
	self:addLuaEvent("ON_THROWTKNIFE_RESULT_CLICK", self, self.onThrowResultClick);
	self:addLuaEvent("ON_THROWTKNIFE_REVICE_CLICK", self, self.onThrowReviceClick);
end

function Lua_ThrowKnifeChallengeWindow:showWin(param)
	self:superCall("showWin", param);

	DataCenter.throwKnifeData.isThrowKnifeTween = true;
	self:initGame();
	self:sComputeThrowKnifeScore(1);
	Lua_CommonUtil.addTimer("Lua_ThrowKnifeTestWindow", 0.01, self, self.onRotateFun, nil, -1, true);
end

function Lua_ThrowKnifeChallengeWindow:showChallengeInfo()
	self.levelTxt.text = LanUtil.formateStr(776005, self.levelIdx);
	self.scoreTxt.text = LanUtil.formateStr(776006, self.totalScore);
end

function Lua_ThrowKnifeChallengeWindow:updateChallengeInfo(msg)
	self.totalScore = msg.latest_score;
	self.reviveTimes = msg.times;
	self:showChallengeInfo();
end

function Lua_ThrowKnifeChallengeWindow:initGame()
	self.fruitMaxAngle = DataCenter.throwKnifeData.itemMinMinusAngle;
	self.knifeMaxAngle = DataCenter.throwKnifeData.knifeMinMinusAngle;
	self.levelIdx = self.showData.isReStart and 0 or DataCenter.throwKnifeData.saveLevel;
	self.throwKnifeNum = 0;
	self.curIdx = 0;
	self.isFail = false;
	self.remainTime = getTime();
	self.idxArr = Array();

	self.totalScore = 0;
	if not self.showData.isReStart then
		local levelInfoList = DataCenter.throwKnifeData.levelInfoList;
		for i = 1, self.levelIdx do
			self.totalScore = self.totalScore + levelInfoList[i].knife_score * levelInfoList[i].knife_num;
		end
	end

	self.radius = 226;
	self._angle = 0;
	self.startPosY = -300;
	self.distance = self.turntableImg.posY - self.radius - self.startPosY;
	self.compareRadius = self.turntableImg.width / 4 + self.radius / 2;
	self.recordInfoArr = Array(); -- iconType：1刀，2水果，resultAngle：停止时的角度
	
	self.isClick = true;
	self.isNewLevel = true; -- 是否是新一关
	self.isPlayTween = false;
	self.moveKnifeTime = DataCenter.throwKnifeData.knifeTime;
	self.intervalTime = 0.1; -- 两把刀间隔时间
	self:createKnifeFun();

	self.isCanClick = true;
end

function Lua_ThrowKnifeChallengeWindow:onRotateFun()
	local time = getTime();
	local dt = time - self.remainTime;
	self.remainTime = time;

	local changeAngle = dt * self.rotateSpeed;
	self.angle = self.angle + changeAngle * self.direction;

	if not self.isPlayTween then -- 播放进入动画和成功通关不进行计算
		self:updateRotateAndDirection(dt);
		if self.isThrowKnife then
			local angle = self.angle;
			local idx, moveTime, moveDistance, moveY, isResult, resultAngle, itemPosX, move;
			for i = 1, self.idxArr:getSize() do
				idx = self.idxArr[i];
				moveTime = self.recordInfoArr[idx].moveTime;
				moveTime = moveTime - dt;
				self.recordInfoArr[idx].moveTime = moveTime;

				moveDistance = 1 - moveTime / self.moveKnifeTime;
				moveY = self.distance * moveDistance;
				self.recordInfoArr[idx]:setLocation(0, self.startPosY + moveY, 0);
				if moveTime <= 0 then
					isResult = true;

					resultAngle = 270 - angle; -- 刀初始在270度
					resultAngle = resultAngle < 0 and resultAngle + 360 or resultAngle;
					-- itemPosX = self.radius * Mathf.Cos(Mathf.Rad(resultAngle));
					-- itemPosY = self.radius * Mathf.Sin(Mathf.Rad(resultAngle));
					move = self.turntableImg.posY - moveY - self.startPosY;
					move = move < self.compareRadius and self.compareRadius or move;
					itemPosX = move * Mathf.Cos(Mathf.Rad(resultAngle));
					itemPosY = move * Mathf.Sin(Mathf.Rad(resultAngle));
					self.recordInfoArr[idx].knifeState = 2;
					self.recordInfoArr[idx].resultAngle = resultAngle;
					self.recordInfoArr[idx]:setParent(self.turntableImg.mainContent);
					self.recordInfoArr[idx]:setSiblingIndex(idx - 1);
					self.recordInfoArr[idx]:setLocation(itemPosX, itemPosY, 0);
					self.recordInfoArr[idx]:setIconRot(resultAngle + 90);
				end
			end
			if isResult then
				self:judgeResult();
			end
		end
	end
end

function Lua_ThrowKnifeChallengeWindow.Get:angle()
	return self._angle;
end

function Lua_ThrowKnifeChallengeWindow.Set:angle(val)
	if val and self._angle ~= val % 360 then
		self._angle = val % 360;
		if not self.isPlayTween then
			self.turntableImg:setRot(0, 0, val);
		end
	end
end

function Lua_ThrowKnifeChallengeWindow:updateRotateAndDirection(dt)
	-- 顺逆交叉改变方向
	local levelInfo = DataCenter.throwKnifeData.levelInfoList[self.levelIdx];
	local rotateInfo = DataCenter.throwKnifeData:getRotateInfo(levelInfo.rotate_id);
	self.totalDirectionTime = self.totalDirectionTime + dt;
	if self.directionType == 3 and #rotateInfo.change_direction_list > 0 then
		if self.changeDiectionTimes + 1 > #rotateInfo.change_direction_list then
			self.changeDiectionTimes = 0;
			self.totalDirectionTime = 0;
		end
		if self.totalDirectionTime >= rotateInfo.change_direction_list[self.changeDiectionTimes + 1] then
			self.changeDiectionTimes = self.changeDiectionTimes + 1;
			self.direction = self.direction * -1;
		end
	end

	-- 改变速度
	self.totalSpeedTime = self.totalSpeedTime + dt;
	local changeTimePoint = rotateInfo.change_time_list[self.changeSpeedTimes + 1];
	if changeTimePoint and self.totalSpeedTime >= changeTimePoint then
		self.changeSpeedTimes = self.changeSpeedTimes + 1;
		if self.recordPreSpeed == nil then
			self.recordPreSpeed = self.rotateSpeed;
		end
	end
	if self.recordPreSpeed then
		local changeTimePoint = rotateInfo.change_time_list[self.changeSpeedTimes];
		local costTime = rotateInfo.speed_cost_time_list[self.changeSpeedTimes];
		local changeTime = self.totalSpeedTime - changeTimePoint;
		local changeSpeed = rotateInfo.change_speed_list[self.changeSpeedTimes];
		self.rotateSpeed = self.recordPreSpeed + changeTime / costTime * changeSpeed;
		if changeTime > costTime then
			self.rotateSpeed = self.recordPreSpeed + changeSpeed;
			self.recordPreSpeed = nil;
			if self.changeSpeedTimes + 1 > #rotateInfo.change_time_list then
				self.changeSpeedTimes = 0;
				self.totalSpeedTime = 0;
			end
		end
	end
end

-- 是否有射击中的刀
function Lua_ThrowKnifeChallengeWindow.Get:isThrowKnife()
	for i = 1, self.curIdx do
		if self.recordInfoArr[i].knifeState == 1 then
			return true;
		end
	end
	return false;
end

function Lua_ThrowKnifeChallengeWindow:judgeResult()
	local state = 1; -- 成功得分
	local actIdx = 1;
	local hitFruitIdx;
	if self.curIdx > 1 then
		local idx, curItem, compareItem, angleMinus, minMinus;
		for i = 1, self.idxArr:getSize() do
			idx = self.idxArr[actIdx];
			curItem = self.recordInfoArr[idx];
			actIdx = actIdx + 1;
			if curItem.knifeState == 2 then
				actIdx = actIdx - 1;
				self.idxArr:shift();
			elseif curItem.knifeState == 1 then
				break;
			end
			for j = 1, idx - 1 do
				compareItem = self.recordInfoArr[j];
				if compareItem.isActive then
					angleMinus = Mathf.Abs(curItem.resultAngle - compareItem.resultAngle);
					angleMinus = angleMinus > 180 and 360 - angleMinus or angleMinus;
					if minMinus == nil or minMinus > angleMinus then
						minMinus = angleMinus;
						if compareItem.iconType == 1 then -- 障碍物
							if minMinus <= self.knifeMaxAngle then
								curItem.isFail = true;
								state = 0; -- 失败
								break;
							end
						else
							if minMinus <= self.fruitMaxAngle + self.knifeMaxAngle then
								state = 2; -- 成功得水果分
								hitFruitIdx = j; -- 击中水果
							end
						end
					end
				end
			end
			if state == 0 then
				break;
			end
		end
	end

	-- 播放闪光特效
	self.turntableImg:setScale(1.05, 1.05, 1);
	TweenUtil.getInstance():addTweenTargetScale(self.turntableImg.mainContent, 0.1, 0, 1, 1, 1);

	if state ~= 0 and hitFruitIdx then
		self.recordInfoArr[hitFruitIdx]:setEnable(false);
		self:playFruitEffFun();
	end
	if state ~= 0 then
		local levelInfoList = DataCenter.throwKnifeData.levelInfoList;
		local levelInfo = levelInfoList[self.levelIdx];
		local totalNum = #levelInfo.target_knife + #levelInfo.target_item + levelInfo.knife_num;
		if state == 1 then
			self.totalScore = self.totalScore + levelInfo.knife_score;
		else
			self.totalScore = self.totalScore + levelInfo.reward_score + levelInfo.knife_score;
		end
		self:showChallengeInfo();

		if self.recordInfoArr[totalNum] and self.recordInfoArr[totalNum].knifeState == 2 then
			if #levelInfoList > self.levelIdx then
				self.isPlayTween = true;
				Lua_CommonUtil.delayCall(0.4, self, self.onDelayShowResultFun, totalNum);
			else
				self:onShowResultFun(1);
				self:sComputeThrowKnifeScore(5);
				Lua_CommonUtil.removeTimer(self, self.onRotateFun, true); -- 通关
			end
		end
	else
		self:showChallengeInfo();
		self.isFail = true;
		Lua_CommonUtil.removeTimer(self, self.onRotateFun, true);
		Lua_CommonUtil.delayCall(1, self, self.onDelayFailFun);
	end
end

function Lua_ThrowKnifeChallengeWindow:playFruitEffFun()
	if self.fruitEff == nil then
		self.fruitEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_sg_01", "ui_jfcd_sg_01", 0, 50);
	end
	self.fruitEff:setEnable(false);
	self.fruitEff:setEnable(true);
end

function Lua_ThrowKnifeChallengeWindow:onDelayShowResultFun(totalNum)
	self.isPlayTween = false;

	self:sComputeThrowKnifeScore(2);
	self.isNewLevel = true;
	self.throwKnifeNum = 0;
	self:playTKLeftTween(totalNum);
	Lua_CommonUtil.removeTimer(self, self.onDelayShowResultFun);
end

function Lua_ThrowKnifeChallengeWindow:onDelayFailFun()
	Lua_CommonUtil.removeTimer(self, self.onDelayFailFun);
	self:onShowResultFun(0);
end

function Lua_ThrowKnifeChallengeWindow:sComputeThrowKnifeScore(state)
	local knifeScore = 0;
	local fruitScore = 0;
	local levelIdx = self.levelIdx;
	local knifeAngleArr = Array();
	if state ~= 1 then
		if self.throwKnifeNum > 0 then -- 要判断是否扔过小刀
			local levelInfo = DataCenter.throwKnifeData.levelInfoList[self.levelIdx];
			local knifeNum = #levelInfo.target_knife; -- 靶子上小刀
			local fruitNum = #levelInfo.target_item; -- 靶子上小刀
			local endIdx = self.throwKnifeNum + knifeNum + fruitNum;
			for i = knifeNum + 1, endIdx do
				if not self.recordInfoArr[i].isFail then
					if self.recordInfoArr[i].iconType == 1 then
						knifeAngleArr:add(self.recordInfoArr[i].resultAngle);
						knifeScore = knifeScore + levelInfo.knife_score;
					else
						if not self.recordInfoArr[i].isActive then
							fruitScore = fruitScore + levelInfo.reward_score;
						end
					end
				else
					break;
				end
			end
		end
	else
		if DataCenter.throwKnifeData.saveLevel > 0 and not self.showData.isReStart then
			levelIdx = DataCenter.throwKnifeData.saveLevel; -- 服务端要拿初始分数
		end
	end
	knifeAngleArr:sort(nil, false);
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sComputeThrowKnifeScore(self.showData.actVo.actIdx,
		state, levelIdx, knifeScore, fruitScore, knifeAngleArr);
end

-- 先向左移动并缩小，超出边界后从右边界往左移动并变大
function Lua_ThrowKnifeChallengeWindow:playTKLeftTween(totalNum)
	self.isCanClick = false;
	self.isPlayTween = true;
	if self.knifeEffArr == nil then
		self.knifeEffArr = Array();
	end

	self.turntableImg:setEnable(false);
	local effPosX, effPosY, angle;
	for i = 1, totalNum do
		effPosX = self.turntableImg.posX + self.radius * Mathf.Cos(Mathf.Rad(self.recordInfoArr[i].resultAngle));
		effPosY = self.turntableImg.posY + self.radius * Mathf.Sin(Mathf.Rad(self.recordInfoArr[i].resultAngle));
		knifeEff = self.knifeEffArr[i];
		if knifeEff == nil then
			knifeEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_xd_01", "ui_jfcd_xd_01", 0, 0);
			self.knifeEffArr:add(knifeEff);
		end
		knifeEff:setRot(0, 0, self.recordInfoArr[i].resultAngle + 90);
		knifeEff:setLocation(effPosX, effPosY, 0);
		knifeEff:setEnable(false);
		knifeEff:setEnable(true);
		TweenUtil.getInstance():stop(knifeEff.mainContent);
		TweenUtil.getInstance():addTweenTargetPos(knifeEff.mainContent, 19, 0, 0, -900, 0, self, self.stopPlayEff, 0, false, i);
	end
	if self.targetEff_1 == nil then
		self.targetEff_1 = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_yp_01", "ui_jfcd_yp_01", 0, 269);
		self.targetEff_2 = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_yp_02", "ui_jfcd_yp_02", 0, 269);
		self.targetEff_3 = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_yp_03", "ui_jfcd_yp_03", 0, 269);
	end
	self.targetEff_1.siblingIndex = self.knifeEffArr[totalNum].siblingIndex + 1;
	self.targetEff_1:setEnable(false);
	self.targetEff_1:setEnable(true);
	self.targetEff_2.siblingIndex = self.targetEff_1.siblingIndex + 1;
	self.targetEff_2:setEnable(false);
	self.targetEff_2:setEnable(true);
	self.targetEff_3.siblingIndex = self.targetEff_2.siblingIndex + 1;
	self.targetEff_3:setEnable(false);
	self.targetEff_3:setEnable(true);
	Lua_CommonUtil.delayCall(1, self, self.playTKRightTween);
end

function Lua_ThrowKnifeChallengeWindow:stopPlayEff(idx)
	TweenUtil.getInstance():stop(self.knifeEffArr[idx].mainContent);
end

function Lua_ThrowKnifeChallengeWindow:playTKRightTween()
	self:createKnifeFun();
	self.turntableImg:setEnable(true);
	self.turntableImg:setLocation(720, 269, 0);
	TweenUtil.getInstance():addTween(self.turntableImg.mainContent,
		1, 0, 0, 269, 0, 0, 0, 0, 1, 1, 1, self, self.stopTween);
end

function Lua_ThrowKnifeChallengeWindow:stopTween()
	self.angle = 0;
	self.isPlayTween = false;
	self.isCanClick = true;
	TweenUtil.getInstance():stop(self.turntableImg.mainContent);
	self:showChallengeInfo();
end

function Lua_ThrowKnifeChallengeWindow:playFruitEffFun()
	if self.fruitEff == nil then
		self.fruitEff = UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_jfcd_sg_01", "ui_jfcd_sg_01", 0, 50);
	end
	self.fruitEff:setEnable(false);
	self.fruitEff:setEnable(true);
end

-- -- 先向左移动并缩小，超出边界后从右边界往左移动并变大
-- function Lua_ThrowKnifeChallengeWindow:playTKLeftTween()
-- 	self.isCanClick = false;
-- 	self.isPlayTween = true;
-- 	TweenUtil.getInstance():addTween(self.turntableImg.mainContent,
-- 		1, 0.2, -720, 269, 0, 0, 0, self.angle, 0.1, 0.1, 1, self, self.playTKRightTween, 0, true);
-- end

-- function Lua_ThrowKnifeChallengeWindow:playTKRightTween()
-- 	self:createKnifeFun();
-- 	self.turntableImg:setLocation(720, 269, 0);
-- 	TweenUtil.getInstance():addTween(self.turntableImg.mainContent,
-- 		1, 0, 0, 269, 0, 0, 0, 0, 1, 1, 1, self, self.stopTween);
-- end

-- function Lua_ThrowKnifeChallengeWindow:stopTween()
-- 	self.angle = 0;
-- 	self.isPlayTween = false;
-- 	self.isCanClick = true;
-- 	TweenUtil.getInstance():stop(self.turntableImg.mainContent);
-- 	self:showChallengeInfo();
-- end

function Lua_ThrowKnifeChallengeWindow:createKnifeFun()
	-- 创建障碍物或水果
	local isNewLevel = self.isNewLevel;
	if self.isNewLevel then
		self.isNewLevel = false;
		self.levelIdx = self.levelIdx + 1;

		self.totalDirectionTime = 0; -- 用于记录时间，每一关不同时间段有不同的旋转方向
		self.totalSpeedTime = 0; -- 用于记录时间，每一关不同时间段有不同的旋转速度
		self.changeSpeedTimes = 0;
		self.changeDiectionTimes = 0;
		self.recordPreSpeed = nil;
		self.rotateSpeed = 50;
		self.direction = 1;

		local levelInfo = DataCenter.throwKnifeData.levelInfoList[self.levelIdx];
		self.turntableImg:setUrl("/images/activities/xui_jfcd_widget_" .. levelInfo.target_type..".png");
		self.upImg:setUrl("/images/activities/xui_jfcd_widget_" .. levelInfo.target_type..".png");
		self.timesTxt.text = LanUtil.formateStr(776007, levelInfo.knife_num);

		local rotateInfo = DataCenter.throwKnifeData:getRotateInfo(levelInfo.rotate_id);
		if rotateInfo then
			self.rotateSpeed = rotateInfo.start_speed;
			self.directionType = rotateInfo.direction;
			if rotateInfo.direction == 1 then -- 顺时针
				self.direction = -1;
			elseif rotateInfo.direction == 2 then -- 逆时针
				self.direction = 1;
			elseif rotateInfo.direction == 3 then -- 顺逆交叉
				self.direction = rotateInfo.start_direction == 1 and -1 or 1;
			end
		end
		self:createNewRoundInfo();
		self.recordTime = getTime();
	end

	-- 创建小刀
	self.curIdx = self.curIdx + 1;
	local item = self.recordInfoArr[self.curIdx];
	if item == nil then
		item = Lua_ThrowKnifeChallengeItem.new(self.content);
		item.contentName = "Lua_ThrowKnifeChallengeItem_" .. self.curIdx;
		self.recordInfoArr:add(item);
	end
	item.knifeState = 0;
	item.isFail = false;
	item.moveTime = self.moveKnifeTime;
	item:setParent(self.content);
	item:setSiblingIndex(1);
	item:setLocation(0, self.startPosY, 0);
	item:setData(true, 270);
	item:setEnable(true);
end

function Lua_ThrowKnifeChallengeWindow:createNewRoundInfo()
	local levelInfo = DataCenter.throwKnifeData.levelInfoList[self.levelIdx];
	local knifeList = levelInfo.target_knife; -- 靶子上小刀
	local fruitList = levelInfo.target_item; -- 靶子上水果
	local dataLen = #knifeList + #fruitList;
	local arrLen = self.recordInfoArr:getSize();
	local maxLen = dataLen > arrLen and dataLen or arrLen;
	local showAngle, item, itemPosX, itemPosY;
	self.curIdx = 0;
	for i = 1, maxLen do
		item = self.recordInfoArr[i];
		if i > dataLen then
			if item ~= nil then
				item.knifeState = 0;
				item:setEnable(false);
			end
		else
			self.curIdx = self.curIdx + 1;

			if i > #knifeList then
				showAngle = fruitList[i - #knifeList];
			else
				showAngle = knifeList[i];
			end
			itemPosX = self.radius * Mathf.Cos(Mathf.Rad(showAngle));
			itemPosY = self.radius * Mathf.Sin(Mathf.Rad(showAngle));

			item = self.recordInfoArr[i];
			if item == nil then
				item = Lua_ThrowKnifeChallengeItem.new(self.turntableImg.mainContent);
				item.contentName = "Lua_ThrowKnifeChallengeItem_" .. i;
				self.recordInfoArr:add(item);
			end
			item.knifeState = 0;
			item.isFail = false;
			item.moveTime = self.moveKnifeTime;
			item:setParent(self.turntableImg.mainContent);
			item:setSiblingIndex(i - 1);
			item:setLocation(itemPosX, itemPosY, 0);
			item:setData(i <= #knifeList, showAngle);
			item:setEnable(true);
		end
	end
end

function Lua_ThrowKnifeChallengeWindow:onThrowClick(go)
	local levelInfo = DataCenter.throwKnifeData.levelInfoList[self.levelIdx];
	local newTime = getTime();
	if not self.isFail and levelInfo.knife_num >= self.throwKnifeNum + 1
		and newTime - self.recordTime > self.intervalTime and self.isCanClick then
			self.recordTime = newTime;
			self.recordInfoArr[self.curIdx].knifeState = 1;
			self.idxArr:add(self.curIdx);

			self.throwKnifeNum = self.throwKnifeNum + 1;
			self.timesTxt.text = LanUtil.formateStr(776007, levelInfo.knife_num - self.throwKnifeNum);
			if self.curIdx < #levelInfo.target_knife + #levelInfo.target_item + levelInfo.knife_num then
				self:createKnifeFun();
			end
	end
end

function Lua_ThrowKnifeChallengeWindow:onThrowResultClick(state)
	self:sComputeThrowKnifeScore(state == 1 and 5 or 4);
	self:hide();
end

function Lua_ThrowKnifeChallengeWindow:onThrowReviceClick()
	self:sComputeThrowKnifeScore(3);

	self.isFail = false;
	self.isNewLevel = true;
	self.levelIdx = self.levelIdx - 1;
	self.throwKnifeNum = 0;
	self.idxArr:clear();
	self:createKnifeFun();

	self.angle = 0;
	Lua_CommonUtil.addTimer("Lua_ThrowKnifeTestWindow", 0.01, self, self.onRotateFun, nil, -1, true);
end

function Lua_ThrowKnifeChallengeWindow:onBackBtnClick()
	local tipsData = {
		owner = self,
		des = LanUtil.getValue(776008);
		btnType = 1,
		cancelLabel = LanUtil.getValue(776009),
		sureLabel = LanUtil.getValue(776010),
		cancelFun = self.onCancelFun,
	};
	Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_ThrowKnifeChallengeWindow:onCancelFun()
	self:onShowResultFun(2);
end

-- 0失败，1通关，2结算
function Lua_ThrowKnifeChallengeWindow:onShowResultFun(state)
	Lua_WindowUtil.openWin("Lua_ThrowKnifeResultWindow", Lua_WinParam.ACTION_OPEN, {
		reviveTimes = self.reviveTimes,
		result = self.totalScore,
		state = state
	});
end

function Lua_ThrowKnifeChallengeWindow:dispose()
	if self.recordInfoArr ~= nil then
		self.recordInfoArr:dispose();
		self.recordInfoArr:clear();
		self.recordInfoArr = nil;
	end
	if self.fruitEff ~= nil then
		self.fruitEff:dispose();
		self.fruitEff = nil;
	end
	if self.knifeEffArr ~= nil then
		for i = 1, self.knifeEffArr:getSize() do
			TweenUtil.getInstance():stop(self.knifeEffArr[i].mainContent);
			self.knifeEffArr[i]:dispose()
		end
		self.knifeEffArr:clear();
		self.knifeEffArr = nil;
	end
	if self.targetEff_1 ~= nil then
		self.targetEff_1:dispose();
		self.targetEff_1 = nil;
	end
	if self.targetEff_2 ~= nil then
		self.targetEff_2:dispose();
		self.targetEff_2 = nil;
	end
	if self.targetEff_3 ~= nil then
		self.targetEff_3:dispose();
		self.targetEff_3 = nil;
	end
	TweenUtil.getInstance():stop(self.turntableImg.mainContent);
	Lua_CommonUtil.removeTimer(self, self.onRotateFun, true);
	Lua_CommonUtil.removeTimer(self, self.onDelayShowResultFun);
	Lua_CommonUtil.removeTimer(self, self.onDelayFailFun);
	DataCenter.throwKnifeData.isThrowKnifeTween = false;
	self:superCall("dispose");
end










Lua_ThrowKnifeChallengeItem = class("Lua_ThrowKnifeChallengeItem", Lua_DisplayObject);
function Lua_ThrowKnifeChallengeItem:ctor(p)
	self:superCall("ctor", p);
end

function Lua_ThrowKnifeChallengeItem:buildShowItem()
	self:superCall("buildShowItem");
	self.icon = UICreate.createUIAtlasImage(self.content, nil, nil, 0, 0, 0, 0);
end

function Lua_ThrowKnifeChallengeItem:setData(isKnife, angle)
	self.iconType = isKnife and 1 or 2;
	self.resultAngle = angle;

	local showAngle;
	if self.iconType == 1 then
		showAngle = angle + 90; -- 刀初始显示为90度
		self.icon:setAltas("throw_knife_atlas", "xui_jfcd_widget_6");
	else
		showAngle = angle - 90; -- 水果初始显示为-90度
		self.icon:setAltas("throw_knife_atlas", "xui_jfcd_widget_4");
	end
	self.icon:resetSize();
	self:setIconRot(showAngle);
end

function Lua_ThrowKnifeChallengeItem:setIconRot(angle)
	self.icon:setRot(0, 0, angle);
end

function Lua_ThrowKnifeChallengeItem.Get:iconType()
	return self._iconType or 1;
end

function Lua_ThrowKnifeChallengeItem.Set:iconType(val)
	self._iconType = val;
end

function Lua_ThrowKnifeChallengeItem.Get:moveTime()
	return self._moveTime or 0;
end

function Lua_ThrowKnifeChallengeItem.Set:moveTime(val)
	self._moveTime = val;
end

-- 0未射击，1射击中，2射中靶子
function Lua_ThrowKnifeChallengeItem.Get:knifeState()
	return self._knifeState or 0;
end

function Lua_ThrowKnifeChallengeItem.Set:knifeState(val)
	self._knifeState = val;
end

function Lua_ThrowKnifeChallengeItem.Get:isFail()
	return self._isFail or false;
end

function Lua_ThrowKnifeChallengeItem.Set:isFail(val)
	self._isFail = val;
end

function Lua_ThrowKnifeChallengeItem.Get:resultAngle()
	return self._resultAngle or 0;
end

function Lua_ThrowKnifeChallengeItem.Set:resultAngle(val)
	if self._resultAngle ~= val then
		self._resultAngle = val;
	end
end

function Lua_ThrowKnifeChallengeItem:dispose()
	if self.icon ~= nil then
		self.icon:dispose();
		self.icon = nil;
	end
	self:superCall("dispose");
end