-- 见缝插刀
Lua_ThrowKnifePanel = class("Lua_ThrowKnifePanel", Lua_DisplayObject);
function Lua_ThrowKnifePanel:ctor(p, skinName)
	self:superCall("ctor", p, skinName);
end

function Lua_ThrowKnifePanel:buildShowItem()
	self:superCall("buildShowItem");

	self.rectW = 530;
	self.rectH = 177;
	self.subRectW = self.rectW;
	self.subRectH = 177;
	self.subMinPos = -(self.subRectW - self.rectW) / 2;
	self.subMaxPox = -self.subMinPos;
	self.spaceW = 88;
	self.stageArr = Array();

	self.taskRedPoint = Lua_RedPointIcon.new(self.content, Lua_RedPointList.ThrowKnife_Task);
	self.taskRedPoint:setRectAnchor(1);
	self.taskRedPoint:setLocation(330, -190, 0);

	self.scrollContainer = UICreate.createScrollContainer(self.content, 0, 0, self.rectW, self.rectH);
	self.scrollContainer.horizontal = true;
	self.scrollContainer.vertical = false;
	self.scrollContainer.movementTypeValue = 1;
	self.scrollContainer.elasticity = 0.3;
	self.scrollContainer.inertia = true;
	self.scrollContainer.siblingIndex = 8;
	self.scrollContainer.anchorValue = 7;
	self.scrollContainer:setScrollAnchor(0.5, 0.5, 0.5, 0.5);
	self.scrollContainer:setScrollPivot(4);
	self.scrollContainer:addRectMask2D();
	self.scrollContainer:addMousDownFuntion(self, self.onMouseDownFun);
	self.scrollContainer:addMouseUpFuntion(self, self.onMouseUpFun);
	self.scrollContainer:setLocation(52, 506.5, 0);

	self.sub = self.scrollContainer.scrollContent;
	self.scoreBar = UICreate.createProgressBar(self.sub, "throw_knife_atlas", "xui_jfcd_Progress _01_b",
		"throw_knife_atlas", "xui_jfcd_Progress _01_f", -self.rectW / 2, 0, self.rectW, 10);
	self.scoreBar.pivot = Lua_WindowUtil.getPivot(3);

	self.ruleBtn:addClickFuntion(self, self.onRuleClick);
	self.taskBtn:addClickFuntion(self, self.onTaskClick);
	self.rankBtn:addClickFuntion(self, self.onRankClick);
	self.challengeBtn:addClickFuntion(self, self.onChallengeClick);

	self:addLuaEvent("UPDATE_THROWTKNIFE_INFO", self, self.updateThrowKnifeInfo);
	self:addLuaEvent("ON_THROWTKNIFE_REFRESH_CLICK", self, self.onThrowKnifeRefreshClick);
	self:addLuaEvent("ON_THROWTKNIFE_PROGRESS_CLICK", self, self.onThrowKnifeProgressClick);
end

function Lua_ThrowKnifePanel:onScreenResize(adaptHeight)
	self:superCall("onScreenResize", adaptHeight);
end

function Lua_ThrowKnifePanel:setData(actVo)
	self.actVo = actVo;
	self.titleTxt.text = actVo.title;
	self:startTimer();
	self:updateLeftTime();

	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sThrowKnifeTaskInfo(actVo.actIdx, 1); -- 每日任务红点
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sThrowKnifeTaskInfo(actVo.actIdx, 2); -- 累计任务红点
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sThrowKnifeLevelInfo(actVo.actIdx); -- 提前获取关卡信息
end

function Lua_ThrowKnifePanel:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self:addTimer("Lua_ThrowKnifePanel", 1, self, self.updateLeftTime);
	end
end

function Lua_ThrowKnifePanel:updateLeftTime()
	local time = self.actVo.residueTime;
	self.leftTimeTxt.text = LanUtil.formateStr(776000, Lua_TimeTransform.transSeconds16(time, true, true, true));
	if time <= 0 then
		self.leftTimeTxt:setLangValue(776001);
		self:endTimer();
	end
end

function Lua_ThrowKnifePanel:endTimer()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.updateLeftTime);
	end
end

function Lua_ThrowKnifePanel:updateThrowKnifeInfo(msg)
	local rewardList = msg.progress_reward;
	local dataLen = #rewardList;
	local barWidth = dataLen * self.spaceW;
	self.subRectW = barWidth + 38; -- 38 item的宽/2
	local halfSubRectW = self.subRectW / 2;
	if not self.isFirstIn then
		self.isFirstIn = true;
		self.subMinPos = -(self.subRectW - self.rectW) / 2;
		self.subMaxPox = -self.subMinPos;
		-- self.scrollContainer.horizontal = dataLen > 5;
		self.scrollContainer:setScrollSize(self.subRectW, self.subRectH);
		self.scrollContainer:setScrollPosition(self.subMaxPox, 0, 0);

		self.scoreBar:SetDimensions(barWidth, 6);
		self.scoreBar:setLocation(-halfSubRectW, -55, 0);
	end

	local score = msg.top_grade;
	self.scoreTxt.text = score;

	local vo = rewardList[dataLen];
	local maxValue = tonumber(tostring(vo.cond_int));
	local value = 0;
	local index = 1;
	if score >= maxValue then
		value = maxValue;
	else
		for i = 1, dataLen do
			vo = rewardList[i];
			if tonumber(tostring(vo.cond_int)) >= score then
				index = i;
				break;
			end
		end

		local num;
		local stepSpace = maxValue / dataLen;
		vo = rewardList[index];
		value = value + stepSpace * (index - 1);
		if index <= 1 then
			num = stepSpace / tonumber(tostring(vo.cond_int));
			value = value + score * num;
		else
			local preVo = rewardList[index - 1];
			num = stepSpace / (tonumber(tostring(vo.cond_int)) - tonumber(tostring(preVo.cond_int)));
			value = value + (score - tonumber(tostring(preVo.cond_int))) * num;
		end
	end
	self.scoreBar:setMaxValue(maxValue);
	self.scoreBar:setValue(value);
	
	local hasLen = self.stageArr:getSize();
	local maxLen = dataLen > hasLen and dataLen or hasLen;
	local item;
	local index = 1;
	for i = 1, maxLen do
		item = self.stageArr[i];
		if i > dataLen then
			if item ~= nil then
				item:setEnable(false);
			end
		else
			vo = rewardList[i];
			if vo.state == 2 then index = i end
			if item == nil then
				item = Lua_ThrowKnifeStageRewardItem.new(self.sub);
				item.contentName = "Lua_ThrowKnifeStageRewardItem_" .. i;
				self.stageArr:add(item);
			end
			item:setEnable(true);
			item:setLocation(i / dataLen * barWidth - halfSubRectW, -35, 0);
			item:setData(vo, i == dataLen);
		end
	end
	self.scoreBar.siblingIndex = self.stageArr[dataLen].siblingIndex + 1;
	-- self:moveToIndex(dataLen > 5 and index or 5);
	self:moveToIndex(index);
end

function Lua_ThrowKnifePanel:moveToIndex(idx)
	local targetPos = self.subMaxPox;
	for i = 1, idx - 1 do
		targetPos = targetPos - self.spaceW;
	end

	if targetPos > self.subMaxPox then
		targetPos = self.subMaxPox;
	elseif targetPos < self.subMinPos then
		targetPos = self.subMinPos;
	end
	self.scrollContainer:setScrollPosition(targetPos, 0, 0);
end

function Lua_ThrowKnifePanel:onMouseDownFun()
	self.startMouseX = CommonUtil.mousePositionX;
	self.startMouseY = CommonUtil.mousePositionY;
	self.startClickTime = getTime();
end

function Lua_ThrowKnifePanel:onMouseUpFun()
	local dx = CommonUtil.mousePositionX - self.startMouseX;
	local dy = CommonUtil.mousePositionY - self.startMouseY;
	if getTime() - self.startClickTime <= 0.3 and Mathf.Abs(dx) + Mathf.Abs(dy) < 4 then
		self:checkPressItem();
	end
end

function Lua_ThrowKnifePanel:checkPressItem()
	local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
	local len = self.stageArr:getSize();
	for i = 1, len do
		if self.stageArr and self.stageArr[i] then
			self.stageArr[i]:showOrHideReward(false);
			if self.stageArr[i].isActive and self.stageArr[i]:checkInBound(winPos.x, winPos.y) then
				self.stageArr[i]:onPressClick();
			end
		end
	end
end

function Lua_ThrowKnifePanel:onRuleClick(go)
	local tipData = { des = self.actVo.help_des, posX = 180, posY = -240 };
	Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipData);
end

function Lua_ThrowKnifePanel:onThrowKnifeRefreshClick()
	self:onThrowKnifeOperate(1);
end

function Lua_ThrowKnifePanel:onTaskClick(go)
	self:onThrowKnifeOperate(2);
end

function Lua_ThrowKnifePanel:onRankClick(go)
	self:onThrowKnifeOperate(3);
end

function Lua_ThrowKnifePanel:onChallengeClick()
	self:onThrowKnifeOperate(4);
end

function Lua_ThrowKnifePanel:onThrowKnifeProgressClick(param)
	self:onThrowKnifeOperate(5, param);
end

function Lua_ThrowKnifePanel:onThrowKnifeOperate(operateType, param)
	if DataCenter.backstageActData:checkActIsEnd(self.actVo) then
		return;
	end

	if operateType == 1 then -- 开启刷新
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sActivityInfo(self.actVo.actIdx);
	elseif operateType == 2 then -- 任务界面
		Lua_WindowUtil.openWin("Lua_ThrowKnifeTaskWindow", Lua_WinParam.ACTION_OPEN, self.actVo);
	elseif operateType == 3 then -- 排行界面
		Lua_WindowUtil.openWin("Lua_ThrowKnifeRankWindow", Lua_WinParam.ACTION_OPEN, self.actVo);
	elseif operateType == 4 then -- 挑战界面
		local saveLevel = DataCenter.throwKnifeData.saveLevel;
		if saveLevel > 0 then
			local tipsData = {
				owner = self,
				des = LanUtil.formateStr(776002, saveLevel, saveLevel + 1);
				btnType = 1,
				cancelLabel = LanUtil.getValue(776003),
				sureLabel = LanUtil.getValue(776004),
				cancelFun = self.reStartFun,
				sureFun = self.onChallengeFun
			};
			Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
		else
			self:onChallengeFun();
		end
	elseif operateType == 5 then -- 领取阶段奖励/展示阶段奖励
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sDrawThrowKnifeStageReward(self.actVo.actIdx);
	end
end

function Lua_ThrowKnifePanel:reStartFun()
	self:onChallengeFun(true);
end

function Lua_ThrowKnifePanel:onChallengeFun(isReStart)
	self:onThrowKnifeRefreshClick();
	Lua_WindowUtil.openWin("Lua_ThrowKnifeChallengeWindow", Lua_WinParam.ACTION_OPEN, {actVo = self.actVo, isReStart = isReStart == true});
end

function Lua_ThrowKnifePanel:dispose()
	if self.taskRedPoint ~= nil then
		self.taskRedPoint:dispose();
		self.taskRedPoint = nil;
	end
	if self.scrollContainer ~= nil then
		self.scrollContainer:dispose();
		self.scrollContainer = nil;
	end
	if self.scoreBar ~= nil then
		self.scoreBar:dispose();
		self.scoreBar = nil;
	end
	if self.stageArr ~= nil then
		self.stageArr:dispose();
		self.stageArr:clear();
		self.stageArr = nil;
	end
	self.sub = nil;
	self.actVo = nil;
	self:superCall("dispose");
end










Lua_ThrowKnifeStageRewardItem = class("Lua_ThrowKnifeStageRewardItem", Lua_DisplayObject);
function Lua_ThrowKnifeStageRewardItem:ctor(p)
	self:superCall("ctor", p, "ThrowKnifeStageRewardItem_skin");
end

function Lua_ThrowKnifeStageRewardItem:buildShowItem()
	self:superCall("buildShowItem");

	self:showOrHideReward(false);
	self.itemIcon:addLoadCompleteFun(self, self.onLoadCompletedFun);
end

function Lua_ThrowKnifeStageRewardItem:setData(data, isEnd)
	self._data = data;
	self.isEnd = isEnd;
	if data == nil then
		return;
	end

	self.scoreTxt.text = data.cond_int;

	local stateSprite;
	local pointSprite;
	if data.state == 0 then -- 未达成
		stateSprite = "xui_bfbz_widget_7";
		pointSprite = "xui_bfbz_widget_10";
	elseif data.state == 1 then -- 可领取
		stateSprite = "xui_bfbz_widget_9";
		pointSprite = "xui_bfbz_widget_11";
	else -- 已领取
		stateSprite = "xui_bfbz_widget_8";
		pointSprite = "xui_bfbz_widget_10";
	end
	self.itemIcon:setAltas("crack_shot_atlas", stateSprite);
	self.pointIcon:setAltas("crack_shot_atlas", pointSprite);
	self.drawIcon:setEnable(data.state == 2);
	self:showRedDot(data.state == 1);
end

function Lua_ThrowKnifeStageRewardItem:showRedDot(bShow)
	if bShow == true then
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content);
			self.redPoint:createRedPointEff();
			self.redPoint:setScale(0.9, 0.9);
			self.redPoint:setLocation(17, 20, 0);
		end
		self.redPoint:updateRedPointShow(true);
	else
		if self.redPoint ~= nil then
			self.redPoint:updateRedPointShow(false);
		end
	end
end

function Lua_ThrowKnifeStageRewardItem:checkInBound(posX, posY)
	local minX = self.centerPosX - self.itemIcon.width / 2 * self.itemIcon.scaleX;
	local maxX = self.centerPosX + self.itemIcon.width / 2 * self.itemIcon.scaleX;
	local minY = self.centerPosY + self.itemIcon.posY - self.itemIcon.height / 2 * self.itemIcon.scaleY;
	local maxY = self.centerPosY + self.itemIcon.posY + self.itemIcon.height / 2 * self.itemIcon.scaleY;
	return posX >= minX and posX <= maxX and posY >= minY and posY <= maxY;
end

function Lua_ThrowKnifeStageRewardItem:onPressClick(go)
	if self._data.state == 1 then
		Lua_MsgUtil.callFunc("ON_THROWTKNIFE_PROGRESS_CLICK", self._data);
	else
		self:showOrHideReward(true);
		local list = self._data.item_list;
		local scaleX = self.isEnd and -1 or 1;
		local posX = self.isEnd and 38.5 or -38.5;
		self.rewardBg:SetDimensions(#list * 70 + 21, 107);
		self.rewardBg:setScale(scaleX, 1, 1);
		self.rewardBg:setLocation(posX, 0, 0);

		local pivot = self.isEnd and 5 or 3;
		local posX = self.isEnd and 28.5 or -28.5;
		if self.rewardList == nil then
			self.rewardList = Lua_DragList.new(self.rewardObj, 70, 70, Lua_AxisDefine.Vertical);
			self.rewardList.canDrag = false;
			self.rewardList:setListCell(Lua_ThrowKnifeProgressRewardItem, 70, 70, 999);
		end
		self.rewardList:setContentRect(#list * 70, 70);
		self.rewardList:setArrayData(list);
		self.rewardList:setRectPivot(pivot);
		self.rewardList:setLocation(posX, 7, 0);
	end
end

function Lua_ThrowKnifeStageRewardItem:showOrHideReward(bShow)
	self.rewardObj:SetActive(bShow);
end

function Lua_ThrowKnifeStageRewardItem:onLoadCompletedFun(go)
	self.itemIcon:resetSize();
end

function Lua_ThrowKnifeStageRewardItem:dispose()
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	if self.rewardList ~= nil then
		self.rewardList:dispose();
		self.rewardList = nil;
	end
	self._data = nil;
	self:superCall("dispose");
end










Lua_ThrowKnifeProgressRewardItem = class("Lua_ThrowKnifeProgressRewardItem", Lua_ItemIconCell);
function Lua_ThrowKnifeProgressRewardItem:buildShowItem()
	self:superCall("buildShowItem");
	self.itemIcon:setScale(0.55, 0.55);
end