-- 古树秘宝-商店界面
Lua_TreeTreasureShopWindow = class("Lua_TreeTreasureShopWindow", Lua_Window);
function Lua_TreeTreasureShopWindow:ctor()
	self:superCall("ctor");
	
	self.windowName = "Lua_TreeTreasureShopWindow";
	self.skinName = "TreeTreasureShopWindow_skin";
	self.layer = MainWinDefine.NORMAL_WINDOW_LAYER;
	self.isFullSizeWin = true;
	self.showWinBg = true;
	self.winBgPath = "/images/activities/xui_gsmb_bg_3.jpg";
	self:addPreLoadRes(self.winBgPath, Lua_LoadFileType.IMG);
end

function Lua_TreeTreasureShopWindow:buildShowItem()
	self:superCall("buildShowItem");

	self.shopList = Lua_DragList.new(self.content, 675, 962 + self.adaptHeight, Lua_AxisDefine.Vertical);
	self.shopList:setListCell(Lua_TreeTreasureShopItem, 333, 155, 2);
	self.shopList:setCellSpacing(9, 3);
	self.shopList:setLocation(0, -121, 0);

	self.addBtn:addClickFuntion(self, self.onAddClick);

	self:addLuaEvent("UPDATE_TREETREASURE_SHOP_INFO", self, self.updateTreeTreasureShopInfo);
	self:addLuaEvent("MONEY_UPDATE", self, self.onItemUpdate); 
	self:addLuaEvent("PACK_ITEMS_UPDATE", self, self.onItemUpdate);
	self:addLuaEvent("ON_TREETREASURE_SHOP_ITEMCLICK", self, self.onBuyClick);
end

function Lua_TreeTreasureShopWindow:showWin(param)
	self:superCall("showWin", param);

	self:startTimer();
	self:updateLeftTime();
	Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sTreeTreasureShopInfo(param.actIdx);
end

function Lua_TreeTreasureShopWindow:onScreenResize()
	self:superCall("onScreenResize");
	self.shopList:setContentRect(675, 962 + self.adaptHeight);
end

function Lua_TreeTreasureShopWindow:startTimer()
	if self.bTimer ~= true then
		self.bTimer = true;
		self:addTimer("Lua_TreeTreasureShopWindow", 1, self, self.updateLeftTime);
	end
end

function Lua_TreeTreasureShopWindow:updateLeftTime()
	local time = self.showData.residueTime;
	self.timeTxt.text = LanUtil.formateStr(797008, Lua_TimeTransform.transSeconds16(time, true, true, true));
	if time <= 0 then
		self.timeTxt:setLangValue(797009);
		self:stopTimer();
	end
end

function Lua_TreeTreasureShopWindow:stopTimer()
	if self.bTimer == true then
		self.bTimer = false;
		self:removeTimer(self, self.updateLeftTime);
	end
end

function Lua_TreeTreasureShopWindow:updateTreeTreasureShopInfo(msg)
	if msg == nil then
		return
	end

	local tvo = Lua_ToolVo.createToolVo(DataCenter.treeTreasureData.shopCostId);
	self.itemIcon:setAltas(tvo.atlas, tvo.sprite);
	self.itemTxt.text = numberTrans(DataCenter.packData:getItemCountById(DataCenter.treeTreasureData.shopCostId));
	self.shopList:setArrayData(msg.cash_item);
end

function Lua_TreeTreasureShopWindow:onItemUpdate(dic)
	if dic and dic[DataCenter.treeTreasureData.shopCostId] then
		self.shopList:updateShowList();
	end
end

function Lua_TreeTreasureShopWindow:onAddClick(go)
	if DataCenter.treeTreasureData.shopCostId then
		DataCenter.packUtil:openPathWindow(DataCenter.treeTreasureData.shopCostId, false);
	end
end

function Lua_TreeTreasureShopWindow:onBuyClick(data)
	if DataCenter.backstageActData:checkActIsEnd(self.showData) then
		self:hide();
		return;
	end
	self.itemData = data;

	local param = {
		id = data.item_id, -- 购买道具id
		itemCount = data.amount, -- 购买一份商品所携带的数量
		showStyle = 2,
		limitCount = data.limit_times - data.times, -- 限购数量
		costId = DataCenter.treeTreasureData.shopCostId, -- 消耗道具id
		costCount = data.recharge, -- 消耗道具数量
		tipStr = "",
		isShowTipWindow = true, -- 点击购买按钮时，是否显示提示窗口
		isShowBackBtn = false, -- 是否显示返回按钮
		cancelBtnStr = LanUtil.getValue(125051), -- 默认取消
		sureBtnStr = LanUtil.getValue(125052), -- 默认确定
		owner = self, 
		callFunc = self.onSureBuyFun, -- 回调事件
		priceStyle = 1, -- 价格显示类型
		font_size = 26, -- tipWindow文本字体大小
		iconWidth = 46,
		iconHeight = 46
	}; 
	Lua_WindowUtil.openWin("Lua_ShoppingGoodsWindow", Lua_WinParam.ACTION_OPEN, param);
end

function Lua_TreeTreasureShopWindow:onSureBuyFun(id, selectNum)
	if self.itemData then
		Lua_ProxyManager.getProxy("Lua_OperationalActivitiesProxy"):sBuyTreeTreasureShopItem(self.showData.actIdx, self.itemData.id, selectNum);
	end
	self.itemData = nil;
end

function Lua_TreeTreasureShopWindow:dispose()
	if self.shopList ~= nil then
		self.shopList:dispose();
		self.shopList = nil;
	end
	self.itemData = nil;
	self:stopTimer();
	self:superCall("dispose");
end











Lua_TreeTreasureShopItem = class("Lua_TreeTreasureShopItem", Lua_ListCell);
function Lua_TreeTreasureShopItem:ctor(p)
	self:superCall("ctor", p, "TreeTreasureShopItem_skin");
end

function Lua_TreeTreasureShopItem:buildShowItem()
	self:superCall("buildShowItem");

	self.rewardIcon = Lua_ItemIcon.new(self.content);
	self.rewardIcon:setSiblingIndex(1);
	self.rewardIcon:setLocation(-83, 4, 0);
	self.rewardIcon:setScale(0.95, 0.95, 0.95);
end

function Lua_TreeTreasureShopItem:setData(data)
	self._data = data;
	if data == nil then
		return;
	end

	local tvo = Lua_ToolVo.createToolVo(data.item_id, data.amount);
	self.rewardIcon:setData(tvo);
	self.itemIcon:setAltas(tvo.atlas, tvo.sprite);
	self.nameTxt.text = tvo.name;
	self.nameTxt:resetSize();
	if self.nameTxt.width > 150 then
		if self.scrollText == nil then
			self.scrollText = Lua_ScrollText.new(self.content, Lua_AxisDefine.Horizontal, 150, 24);
			self.scrollText:setLocation(60, 41, 0);
			self.scrollText:setTextStyle(150, 24, Lua_MyUISettings.huakangFontName, 24, "6F5144", 0);
		end
		self.scrollText:setShowTxt(tvo.name);
		self.scrollText:setEnable(true);
		self.nameTxt:setEnable(false);
	else
		if self.scrollText then
			self.scrollText:setEnable(false);
		end
		self.nameTxt:setEnable(true);
	end
	self.limitTxt.text = LanUtil.formateStr(797016, data.limit_times - data.times);
	self.suggestIcon:setEnable(data.recommend == 1);
	self.maskIcon:setEnable(data.limit_times <= data.times);
	self.sellOutIcon:setEnable(data.limit_times <= data.times);
	self:updateData();
end

function Lua_TreeTreasureShopItem:updateData()
	if DataCenter.treeTreasureData.shopCostId == 0 then
		return;
	end

	local packNum = DataCenter.packData:getItemCountById(DataCenter.treeTreasureData.shopCostId);
	local colorStr = packNum >= self._data.recharge and "FFFFFF" or "FF0000";
	self.costTxt.text = Lua_ColorDefine.getColorStr(colorStr, self._data.recharge);
	self:showRedDot(self._data.recommend == 1 and packNum >= self._data.recharge and self._data.limit_times - self._data.times > 0);
end

function Lua_TreeTreasureShopItem:showRedDot(bShow)
	if bShow == true then
		if self.redPoint == nil then
			self.redPoint = Lua_RedPointIcon.new(self.content);
			self.redPoint:createRedPointEff();
			self.redPoint:setLocation(150, 60, 0);
		end
		self.redPoint:updateRedPointShow(true);
	else
		if self.redPoint ~= nil then
			self.redPoint:updateRedPointShow(false);
		end
	end
end

function Lua_TreeTreasureShopItem:onPressClick()
	if self._data then
		local winPos = LuaUtil.getTouchPosInverseObj(self.mainContent);
		if self.rewardIcon:checkInBound(winPos.x, winPos.y) then
			DataCenter.packUtil:showItemInfo(self._data.item_id, false);
			return;
		end
		if self._data.limit_times <= self._data.times then
			OperateAlert.showLangValue(797010);
		else
			local shopCostId = DataCenter.treeTreasureData.shopCostId;
			local packNum = DataCenter.packData:getItemCountById(shopCostId);
			if packNum < self._data.recharge then
				DataCenter.packUtil:openPathWindow(shopCostId);
			else
				Lua_MsgUtil.callFunc("ON_TREETREASURE_SHOP_ITEMCLICK", self._data);
			end
		end
	end
end

function Lua_TreeTreasureShopItem:dispose()
	if self.rewardIcon ~= nil then
		self.rewardIcon:dispose();
		self.rewardIcon = nil;
	end
	if self.scrollText ~= nil then
		self.scrollText:dispose();
		self.scrollText = nil;
	end
	if self.redPoint ~= nil then
		self.redPoint:dispose();
		self.redPoint = nil;
	end
	self._data = nil;
	self:superCall("dispose");
end